#include <iostream>

#include "file/File.hh"

int main (int argc, char** argv) {

	cout << "===== File implementation test stub =====" << endl;

	cout << "Creating a single File object: ";
	File f = File();
	cout << "done" << endl;

	cout << "opening non-existent file /var/tmp/motd_copy for creation: ";
	f = File("/var/tmp/motd_copy");
	cout << "done" << endl;

	cout << "trying to create()";
	FILE* fd = f.create();
	cout << "done, localName() is " << f.localName() << endl;

	cout << "writing some stuff into it..." << endl;

	cout << "Creating a local (file:/etc/motd) File object: ";
	File f2 = File("file:/etc/motd");
	cout << "done" << endl;

	cout << "Local file name (open_local()): " << f2.open_local(false) << endl;

	cout << "open_for_reading(): ";
	FILE* fd2 = f2.open_for_reading(false);
	cout << "done" << endl;

	cout << "reading to file: ";
#define BUFSIZE 1024
	char buffer[BUFSIZE];
	size_t r;
	while (r = fread (buffer, sizeof(char), BUFSIZE, fd2)) {
		fwrite (buffer, sizeof(char), r, fd);
	}
	cout << "done" << endl;

	cout << "trying to close(): ";
	f2.close();
	cout << "done" << endl;

	cout << endl << endl 
	     << "let's do something similar with remote" << endl;

	string tmpfile;
	{
		// this should be deleted at the end of this block
		File newf = File("http://www.freebsd.org/");
		
		cout << "url(): " << newf.url() << ", localName(): " << newf.localName() << endl;
		cout << "trying to open_local(): " << newf.open_local(true) << endl;
		
		tmpfile = newf.localName();
		cout << "head -1 " << tmpfile << endl;
		
		::system((string("head -1 ") + tmpfile).c_str());
	}

	if (File::exist(tmpfile)) {
		cout << "oups! " << tmpfile << " shouldn't exist" << endl;
	} else {
		cout << "temp file " << tmpfile << " properly deleted" << endl;
	}

	cout << "let's try to create a remote file to see if we panic" << endl;
	f = File("http://www.freebsd.org/");

	try {
		fd = f.create();
	} catch (...) {
		cout << "exception caught, create() not implemented" << endl;
	}
	cout << "reading a single line with open_for_reading()" << endl;
	fd = f.open_for_reading(false);
	char *line = fgetln(fd, &r);
	line[r] = '\0';
	cout << "first line: " << string(line) << endl;
	cout << "close()ing: ";
	f.close();
	cout << "done." << endl;
	cout << "localName() left over: " << f.localName() << endl;
}
