/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/bin/tclh/tclh.cc,v 1.10 2002/04/13 20:32:01 antoine Exp $
 */

// Eugene Skepner <eu@qub.com>
// $Id: tclh.cc,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#include <tcl.h>
#include <iostream>

#include "tcl/LibHTclInit_common.hh"

#ifdef HSYSTEM_HUI
#include "tcl/LibHTclInit_hui.hh"
#include "hui/libhhui_init.hh"
#endif

#ifdef HSYSTEM_FILE
#include "tcl/LibHTclInit_file.hh"
#include "file/libhfile_init.hh"
#endif

#ifdef HSYSTEM_DISK
#include "tcl/LibHTclInit_disk.hh"
#include "disk/libhdisk_init.hh"
#endif

#ifdef HSYSTEM_DATABASE
#include "database/libhdatabase_init.hh"
#endif

#ifdef HSYSTEM_SYSINSTALL
#include "tcl/LibHTclInit_sysinstall.hh"
#include "sysinstall/libhsysinstall_init.hh"
#endif

/**
 * Initialize tclh command interpreter.  Statically linked libh
 * subsystems will add their commands here, and set their tag
 * to 1.
 *
 * @param interp	Tcl interpreter structure.
 * @return TCL_ERROR if any of the initializers fail, otherwise TCL_OK.
 */
static int H_Tcl_AppInit(Tcl_Interp* interp)
{
	if (Tcl_Init(interp) == TCL_ERROR)
		return(TCL_ERROR);

	if (Htcl_Init(interp) == TCL_ERROR)
		return(TCL_ERROR);
	Tcl_GlobalEval(interp, const_cast<char*>("set H_libhtcl 1"));

#ifdef HSYSTEM_HUI
	if (Htclhui_Init(interp) == TCL_ERROR)
		return(TCL_ERROR);
	if (Hui_Init(interp) == TCL_ERROR)
		return(TCL_ERROR);
	Tcl_GlobalEval(interp, const_cast<char*>("set H_libhui 1"));
#endif

#ifdef HSYSTEM_FILE
	if (Htclfile_Init(interp) == TCL_ERROR)
		return(TCL_ERROR);
	if (Hfile_Init(interp) == TCL_ERROR)
		return(TCL_ERROR);
	Tcl_GlobalEval(interp, const_cast<char*>("set H_libhfile 1"));
#endif

#ifdef HSYSTEM_DISK
	if (Htcldisk_Init(interp) == TCL_ERROR)
		return(TCL_ERROR);
	if (Hdisk_Init(interp) == TCL_ERROR)
		return(TCL_ERROR);
	Tcl_GlobalEval(interp, const_cast<char*>("set H_libhdisk 1"));
#endif

#ifdef HSYSTEM_DATABASE
	if (Hdatabase_Init(interp) == TCL_ERROR)
		return(TCL_ERROR);
	Tcl_GlobalEval(interp, const_cast<char*>("set H_libhdatabase 1"));
#endif

#ifdef HSYSTEM_SYSINSTALL
	if (Htclsysinstall_Init(interp) == TCL_ERROR)
		return(TCL_ERROR);
	if (Hsysinstall_Init(interp) == TCL_ERROR)
		return(TCL_ERROR);
	Tcl_GlobalEval(interp, const_cast<char*>("set H_libhsysinstall 1"));
#endif

	Tcl_SetVar(interp, const_cast<char*>("tcl_rcFileName"), const_cast<char*>("~/.tclshrc"), TCL_GLOBAL_ONLY);

	return(TCL_OK);

}

/**
 * Start tclh interpreter.
 *
 * The Tclh interpreter is basically a tclsh-like shell but with added
 * features from libh libraries.
 *
 * @see Tcl_Main(1)
 * @see H_Tcl_AppInit()
 */
int
main(int argc, char** argv)
{
	Tcl_Main(argc, argv, &H_Tcl_AppInit);
	return 0;
}
