/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/include/Configuration.hh,v 1.8 2002/04/15 16:21:47 antoine Exp $
 */

// -*- C++ -*-  Wed Dec 23 1998
// Eugene Skepner <eu@qub.com>
// $Id: Configuration.hh,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef Configuration_hh
#define Configuration_hh

#include "LanguageInterface.hh"

/**
 * General libh configuration data.
 *
 * This class represents global configuration data that affects
 * package processing.
 *
 * Any consumer of this class should use Configuration::instance()
 * instead of creating seperate instances.
 */
class Configuration : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  protected:
     /**
      * Initialize all fields to default values.
      *
      * This constructor shouldn't normally be accessed outside this
      * class. To access Configuration objects, instance() should be
      * used instead.
      */
     Configuration();

  public:
     /// Does nothing
     virtual ~Configuration();

     /// return wether we use the prefix specified in the package
     bool usePackagePrefix() const { return mUsePackagePrefix; }
     /// set wether we use the prefix specified in the package
     void usePackagePrefix( bool use ) { mUsePackagePrefix = use; }
     
     /// return the prefix where files are installed 
     const string& prefix() const { return mPrefix; }
     /// set the prefix
     void prefix( const string& aPrefix ) { mPrefix = aPrefix; }

     /// return the path to where installed packages metadata is stored
     const string& dbPath() const { return mDbPath; }
     /// change the path where installed package metadata is stored, use with care
     void dbPath( const string& aDbPath ) { mDbPath = aDbPath; }

     /// return the path to the patch(1) binary
     const string& path_to_patch() const { return mPathToPatch; }
     /// set the path to the patch(1) binary
     void path_to_patch( const string& aPathToPatch ) { mPathToPatch = aPathToPatch; }

     /// return wether no action is taken by the installer
     bool no_action() const { return mNoAction; }
     /// set wether are allowed to do anything at all
     void no_action( bool aNoAction ) { mNoAction = aNoAction; }

     /// return the temporary directory
     const string& tempDir() const { return mTempDir; }
     /// set the temporary directory
     void tempDir( const string& aTempDir ) { mTempDir = aTempDir; }

     /**
      * Return and create (if not done already) an instance of
      * Configuration.
      *
      * This method is used to access the global runtime
      * configuration. There should logically be only one instance of
      * Configuration used at any given time in a running libh program.
      */
     static Pointer<Configuration> instance()
          {
               if ( !sConfiguration )
                    sConfiguration = new Configuration();
               return sConfiguration;
          }

  private:
     bool mUsePackagePrefix;
     string mPrefix;
     string mPathToPatch;
     string mDbPath;
     bool mNoAction;
     string mTempDir;

     static Pointer<Configuration> sConfiguration;

}; // class Configuration

//======================================================================

#endif // Configuration_hh
