/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/include/hui/HuiQt.hh,v 1.37 2002/05/14 08:59:31 okumoto Exp $
 */

// -*- C++ -*-  Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiQt.hh,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiQt_hh
#define HuiQt_hh

#include "Hui.hh"

#include <qapp.h>
#include <qlayout.h>
#include <qlistview.h>

//======================================================================

class QLabel;
class QResizableDialog;
class QResizableMainWindow;
class QPushButtonEnterLeave;
class QLineEditEnterLeave;
class QValidator;
class QButtonGroup;
class QListViewEnterLeave;
class QMultiLineEditEnterLeave;
class QProgressBar;
class QMenuData;

//======================================================================

class HuiQResizable : public QObject
{
	Q_OBJECT;

public slots:
	virtual void resized(const QSize &aSize)
	{
		widget_resized(aSize);
	}

	virtual void enteredLeft(bool aEntered)
	{
		widget_entered_left(aEntered);
	}

protected:
	virtual void widget_resized(const QSize &aSize) = 0;
	virtual void widget_entered_left(bool aEntered) = 0;

}; // class HuiQResizable

//======================================================================

class HuiQtEnterLeave : public QObject
{
	Q_OBJECT;

public:
	HuiQtEnterLeave(Hui::Widget &aWidget)
		: mWidget(aWidget) {}

public slots:
	virtual void enter() { mWidget.enter(); }
	virtual void leave() { mWidget.leave(); }

private:
	Hui::Widget &mWidget;

}; // class HuiQtEnterLeave

//======================================================================

class HuiQtButtonClick : public QObject
{
	Q_OBJECT;

public:
	HuiQtButtonClick(Hui::ButtonWidget &aWidget)
		: mWidget(aWidget) {}

public slots:
	virtual void click() { mWidget.click(); }

private:
	Hui::ButtonWidget &mWidget;

}; // class HuiQtButtonClick

//======================================================================

class HuiQtButtonGroupClick : public QObject
{
	Q_OBJECT;

public:
	HuiQtButtonGroupClick(Hui::ButtonsGroupWidget &aWidget)
		: mWidget(aWidget) {}

public slots:
	virtual void click(int id) { mWidget.click(id); }

private:
	Hui::ButtonsGroupWidget &mWidget;

}; // class HuiQtButtonGroupClick

//======================================================================

class HuiQtMenuClick : public QObject,
		       virtual public Shared
{
	Q_OBJECT;

public:
	HuiQtMenuClick(Hui::MenuItemWidget &aWidget)
		: mWidget(aWidget) {}

public slots:
	virtual void click() { mWidget.click(); }

private:
	Hui::MenuItemWidget &mWidget;

}; // class HuiQtMenuClick

//======================================================================

class HuiQt : public QApplication, public Hui {
    CLASS_METHODS_DESCRIPTIONS_DECLARATION;

protected:
    class GroupWidget;

    class Subwidgets : public list<Widget_P> {
    public:
	/**
	 * Constructor.
	 */
	Subwidgets() {}

	/**
	 * Destructor.
	 */
	~Subwidgets() {}

	Widget_P find(QWidget *p) const;
	void add(Widget_P w);
	void remove(Widget_P w);
    };

    class Widget : virtual public Hui::Widget {
	CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	HuiQtEnterLeave		mHuiQtEnterLeave;
	Pointer<GroupWidget>	mParent;
	QPoint			mPosition;
	QSize			mSize;
	int			mFixedSize;

	friend class HuiQt;

    public:
	/**
	 * Constructor.
	 */
	Widget(Hui &aHui, const string &aName)
	    : Hui::Widget(aHui, aName),
	      mHuiQtEnterLeave(*this)
	{
	    mFixedSize = -1;
	}

	/**
	 * Destructor.
	 */
	virtual ~Widget();

	virtual void show();
	virtual void hide();
	virtual bool hidden() const;

	virtual void disable();
	virtual void enable();
	virtual bool disabled() const;

	virtual void move(const Position &aPosition);
	virtual void move(const QPoint &to);
	virtual Position position() const;

	virtual void resize(const Size &aSize);
	virtual void resize(const QSize &to, int is_arranging = 0);
	virtual Size size() const;

	virtual QWidget *qwidget() = 0;
	virtual const QWidget *qwidget() const = 0;

	void
	parent(Pointer<GroupWidget> aParent)
	{
	    mParent = aParent;
	};

	Pointer<GroupWidget>
	parent()
	{
	    return mParent;
	}

	const Pointer<GroupWidget>
	parent() const
	{
	    return mParent;
	}

	int
	getFixedSize()
	{
	    return (mFixedSize);
	}

	void
	setFixedSize(int to)
	{
	    mFixedSize = to;
	}

	virtual QRect
	translate(const Position &aPosition, const Size &aSize) const;

	virtual QPoint
	translate(const Position &aPosition) const
	{
	    return translate(aPosition, Size()).topLeft();
	}

	virtual QSize
	translate(const Size &aSize) const
	{
	    return translate(Position(), aSize).size();
	}

	virtual Size
	translate(const QSize &aSize) const;

	virtual Position
	translate(const QPoint &aPosition) const;

	HuiQtEnterLeave &
	huiQtEnterLeave()
	{
	    return mHuiQtEnterLeave;
	}

	void
	setposition(const QPoint &aPosition)
	{
	    mPosition = aPosition;
	}

	void
	setsize(const QSize &aSize)
	{
	    mSize = aSize;
	}

	QPoint
	real_position() const
	{
	    return (mPosition);
	}

	QSize
	real_size() const
	{
	    return (mSize);
	}

    };

	class GroupWidget : virtual public Hui::GroupWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		virtual ~GroupWidget();

		virtual LabelWidget_P label(const string &aName,
							const string &aText,
							const Position &aPosition);
		virtual ButtonWidget_P button(const string &aName,
							  const string &aText,
							  Pointer<LanguageInterface::CallbackData> aCallbackData,
							  const string &aButtonType,
							  const Position &aPosition,
							  const string &aComment);
		virtual InputLineWidget_P inputLine(const string &aName,
								const string &aInitialText,
								int aMaxLength,
								const string &aValidator,
								const string &aTitle,
								const Position &aPosition,
								const string &aComment);
		virtual CheckBoxesWidget_P checkBoxes(const string &aName,
								  const vector<string> &aTexts,
								  const vector<bool> &aInitialValues,
								  const string &aTitle,
								  const Position &aPosition,
								  const vector<string> &aComments);
		virtual RadioMenuWidget_P radioMenu(const string &aName,
								const vector<string> &aTexts,
								const int aInitialValue,
								const string &aTitle,
								const Position &aPosition,
								const vector<string> &aComments);
		virtual ListBoxWidget_P listBox(const string &aName,
							    const vector<string> &aTexts,
							    const int aInitiallySelected,
							    const string &aTitle,
							    const Position &aPosition,
							    const string &aComment);
		virtual TextEditWidget_P textEdit(const string &aName,
							      const string &aInitialText,
							      bool aAllowEdit,
							      const string &aTitle,
							      const Position &aPosition,
							      const string &aComment);
		virtual ProgressIndicatorWidget_P progressIndicator(const string &aName,
										float aMin,
										float aMax,
										float aInitial,
										const string &aTitle,
										const Position &aPosition,
										const string &aComment);
		virtual ContainerWidget_P container(const string &aName,
								const Position &aPosition,
								const string &aAlignment,
								const string &aComment);

		virtual void arrange();
		virtual void arrange(const string &aAlignment,
				     bool arrange_new);

		virtual void current(EmbeddedWidget_P aWidget);
		virtual Widget_P current() const;

		virtual QWidget *qwidget() = 0;
		virtual const QWidget *qwidget() const = 0;

		virtual Widget_P byName(const string &aName) const;
		virtual void erase();

		virtual HuiQt &hui() const { return mHui; };

		const Subwidgets &subwidgets() const { return mWidgets; }
		Subwidgets &subwidgets() { return mWidgets; }

		virtual QPoint real_position() = 0;
		virtual QSize real_size() = 0;

	protected:
		GroupWidget(HuiQt &aHui,
			    const string &aName,
			    const Position &aPosition,
			    const Size &aSize);

	private:
		HuiQt &mHui;
		Subwidgets mWidgets;
		int mArrangement;

	}; // class GroupWidget

	class ContainerWidget : public Hui::ContainerWidget,
				public GroupWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		ContainerWidget(HuiQt &aHui,
				Pointer<GroupWidget> aParent,
				const string &aName);
		virtual ~ContainerWidget();

		// Functions, that need to be virtualized
		void show();
		void hide();
		bool hidden() const;

		void disable();
		void enable();
		bool disabled() const;

		void move(const Position &);
		void move(const QPoint &);
		class Position position() const;

		void resize(const Size &);
		void resize(const QSize &, int is_arranging = 0);
		class Size size() const;
		QSize real_size() { return (mSize); }
		QPoint real_position() { return (mPosition); }

		void parent(Pointer<GroupWidget> aParent) { mParent = aParent; };
		const Pointer<GroupWidget> parent() const { return mParent ; }
		Pointer<GroupWidget> parent() { return mParent ; }

		// Refer to parent's qwidget()
		QWidget *qwidget();
		const QWidget *qwidget() const;

		int getFixedSize() { return(mFixedSize); }
		void setFixedSize(int to) { mFixedSize = to; }

	protected:
		QPoint translate(const Position &aPosition) const;
		QSize translate(const Size &aSize) const;
		Position translate(const QPoint &aPosition) const;
		Size translate(const QSize &aSize) const;

	private:
		QPoint mPosition;
		QSize mSize;
		Pointer<GroupWidget> mParent;
		bool mHidden;
		bool mDisabled;
		int mFixedSize;

	}; // ContainerWidget;

	// --------------------------------------------------

	class ToplevelWidget
		: virtual public Hui::ToplevelWidget,
		  public GroupWidget, public HuiQResizable
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		virtual ~ToplevelWidget();

		virtual void title(const string &aTitle);
		virtual string title() const;

	protected:
		ToplevelWidget(HuiQt &aHui,
			       const string &aName,
			       const Position &aPosition,
			       const Size &aSize);

		virtual void widget_resized(const QSize &aSize);
		virtual void widget_entered_left(bool aEntered);

	}; // class ToplevelWidget

	// --------------------------------------------------

	class WindowWidget : public Widget,
			     public ToplevelWidget,
			     public Hui::WindowWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		WindowWidget(HuiQt &aHui,
			     const string &aName,
			     const string &aTitle,
			     const Position &aPosition,
			     const Size &aSize);
		virtual ~WindowWidget();

		virtual void show();
		virtual void hide();
		virtual void resize(const Size &aSize);
		virtual void resize(const QSize &to);

		virtual QWidget *qwidget();
		virtual const QWidget *qwidget() const;

		virtual void recreateMenu(Hui::MenuSubmenuWidget &aMenuData);

		QSize real_size() { return qwidget()->size(); }
		QPoint real_position() { return QPoint(0, 0); }

	protected:
		virtual QRect translate(const Position &aPosition,
					const Size &aSize) const {
			return HuiQt::Widget::translate(aPosition, aSize);
		}
		virtual Size translate(const QSize &aSize) const {
			return HuiQt::Widget::translate(aSize);
		}

	private:
		HuiQt &mHui;
		QResizableMainWindow *mMainWindow;
		list< Pointer<HuiQtMenuClick> > mMenuClicks;

		virtual void recreateMenu(QMenuData *popup,
					  Hui::MenuSubmenuWidget &aMenuData);

	}; // class WindowWidget

	// --------------------------------------------------

	class DialogWidget : public Widget,
			     public ToplevelWidget,
			     public Hui::DialogWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		DialogWidget(HuiQt &aHui,
			     Widget_P aParent,
			     const string &aName,
			     const string &aTitle,
			     const Position &aPosition,
			     const Size &aSize);
		virtual ~DialogWidget();

		virtual const string &result() const;
		virtual const string &exec();

		virtual bool running() const { return mRunning; }
		virtual void endModal(const string &aResult);

		virtual QWidget *qwidget();
		virtual const QWidget *qwidget() const;

		QSize real_size() { return qwidget()->size(); }
		QPoint real_position() { return QPoint(0, 0); }

	protected:
		virtual QRect translate(const Position &aPosition,
					const Size &aSize) const {
			return HuiQt::Widget::translate(aPosition, aSize);
		}
		virtual Size translate(const QSize &aSize) const {
			return HuiQt::Widget::translate(aSize);
		}

		QResizableDialog *qtwindow() { return mWindow; }

	private:
		HuiQt &mHui;
		QResizableDialog *mWindow;
		string mResult;
		bool mRunning;

	}; // class DialogWidget

	// --------------------------------------------------

	class LabelWidget : public Widget, public Hui::LabelWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		LabelWidget(Hui &aHui,
			    Pointer<GroupWidget> aParent,
			    const string &aName,
			    const string &aText);
		virtual ~LabelWidget();

		virtual void text(const string &aText);
		virtual string text() const;

		virtual QWidget *qwidget();
		virtual const QWidget *qwidget() const;

	private:
		QLabel *mLabel;

	}; // class LabelWidget

	// --------------------------------------------------

	class ButtonWidget : public Widget,
			     public Hui::ButtonWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		ButtonWidget(HuiQt &aHui,
			     Pointer<GroupWidget> aParent,
			     const string &aName,
			     const string &aText,
			     Pointer<LanguageInterface::CallbackData> aCallbackData,
			     const string &aButtonType,
			     const string &aComment);
		virtual ~ButtonWidget();

		virtual void text(const string &aText);
		virtual string text() const;

		virtual QWidget *qwidget();
		virtual const QWidget *qwidget() const;

		virtual void callback_completed(const WidgetCallbackData &aCallbackData);

	private:
		QPushButtonEnterLeave *mButton;
		HuiQtButtonClick mButtonClick;
		string mComment;

	}; // class ButtonWidget

	// --------------------------------------------------

	class TitledWidget : public Widget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		TitledWidget(Hui &aHui,
			     Pointer<GroupWidget> aParent,
			     const string &aName,
			     const string &aTitle);

		virtual void title(const string &aTitle);
		virtual string title() const;
		virtual void move(const QPoint &to);
		virtual void resize(const QSize &to, int is_arranging = 0);
		virtual void move(const Position &to) { Widget::move(to); }
		virtual void resize(const Size &to) { Widget::resize(to); }

		QLabel *qtitle() { return mTitle; }
		const QLabel *qtitle() const { return mTitle; }

	private:
		QLabel *mTitle;

	}; // class TitledWidget

	// --------------------------------------------------

	class InputLineWidget : public TitledWidget,
				public Hui::InputLineWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		InputLineWidget(Hui &aHui,
				Pointer<GroupWidget> aParent,
				const string &aName,
				const string &aInitialText,
				int aMaxLength,
				const string &aValidator,
				const string &aTitle,
				const string &aComment);

		virtual void text(const string &aText);
		virtual string text() const;

		virtual void title(const string &aTitle) { TitledWidget::title(aTitle); }
		virtual string title() const { return TitledWidget::title(); }

		virtual QWidget *qwidget();
		virtual const QWidget *qwidget() const;

	private:
		QLineEditEnterLeave *mInputLine;
		QValidator *mValidator;
		string mComment;

	}; // class InputLineWidget

	friend class InputLineWidget;

	// --------------------------------------------------

	class ButtonsGroupWidget : public Widget,
				   virtual public Hui::ButtonsGroupWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		ButtonsGroupWidget(Hui &aHui,
				   Pointer<GroupWidget> aParent,
				   const string &aName,
				   const string &aTitle,
				   const vector<string> &aComments);

		virtual void text(int boxno,
				  const string &aText);
		virtual string text(int boxno) const;
		virtual void title(const string &aTitle);
		virtual string title() const;
		virtual void disable();
		virtual void enable();
		virtual void disable_item(int boxno);
		virtual void enable_item(int boxno);
		virtual bool item_disabled(int boxno) const;
		virtual int numberOfItems() const;


		virtual QWidget *qwidget();
		virtual const QWidget *qwidget() const;

	protected:
		QButtonGroup *boxes() { return mBoxes; }
		const QButtonGroup *boxes() const { return mBoxes; }
		HuiQtButtonGroupClick &click() { return mButtonGroupClick; }

	private:
		QButtonGroup *mBoxes;
		HuiQtButtonGroupClick mButtonGroupClick;
		vector<string> mComments;

	};

public:
    static string stripAccel(const string &aText);
    static QValidator *make_lineedit_validator(const string &aValidatorType);

    // xxxxxxxxxxxxxxxxx
    class CheckBoxesWidget : public ButtonsGroupWidget, public Hui::CheckBoxesWidget {
	CLASS_METHODS_DESCRIPTIONS_DECLARATION;

    public:
	CheckBoxesWidget(
		Hui &aHui,
		Pointer<GroupWidget> aParent,
		const string &aName,
		const vector<string> &aTexts,
		const vector<bool> &aInitialValues,
		const string &aTitle,
		const vector<string> &aComments);

	virtual bool
	state(
		int boxno,
		int aNewState);

	virtual bool state(int boxno) const;
	virtual bool toggle(int boxno);
    };

    class RadioMenuWidget : public ButtonsGroupWidget, public Hui::RadioMenuWidget {
	CLASS_METHODS_DESCRIPTIONS_DECLARATION;

    public:
	RadioMenuWidget(
		Hui &aHui,
		Pointer<GroupWidget> aParent,
		const string &aName,
		const vector<string> &aTexts,
		int aInitialValue,
		const string &aTitle,
		const vector<string> &aComments);

	virtual void set(int itemno);
	virtual int set() const;
    };

    class ListBoxWidget : public TitledWidget, public Hui::ListBoxWidget {
	CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	vector<string>		mTexts;
	string			mTitleText;
	QListViewEnterLeave	*mListBox;
	QListViewItem		*mTopRow;
	string			mComment;

    public:
	ListBoxWidget(
		Hui &aHui,
		Pointer<GroupWidget> aParent,
		const string &aName,
		const vector<string> &aTexts,
		int aInitiallySelected,
		const string &aTitle,
		const string &aComment);

	virtual int numberOfItems() const;
	virtual string text(int aItemNo) const;
	virtual void text(int aItemNo, const string &aText);
	virtual int select() const;
	virtual void select(int aItemNo);
	virtual void add(int aBefore, const string &aText);
	virtual void remove(int aItemNo);
	virtual void removeAll();

	virtual void
	title(const string &aTitle)
	{
	    mTitleText = aTitle;
	    updatetexts();
	}

	virtual string
	title() const
	{
	    return mTitleText;
	}

	virtual QWidget *qwidget();
	virtual const QWidget *qwidget() const;

    protected:
	virtual void updatetexts();
	virtual void updatetitle(const string &aTitle);

	virtual int
	count_columns(
		const vector<string> &aTexts,
		const string &aTitle);

	virtual string
	HuiQt::ListBoxWidget::extract_cell(
		const vector<string>& aTexts,
		int x,
		int y);

    };

    class TextEditWidget : public TitledWidget, public Hui::TextEditWidget {
	CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	QMultiLineEditEnterLeave	*mTextEdit;
	string				mComment;

    public:
	TextEditWidget(
		Hui &aHui,
		Pointer<GroupWidget> aParent,
		const string &aName,
		const string &aInitialText,
		bool aAllowEdit,
		const string &aTitle,
		const string &aComment);

	virtual void text(const string &aText);
	virtual string text() const;

	virtual void
	title(const string & aTitle)
	{
	    TitledWidget::title(aTitle);
	}

	virtual string
	title() const
	{
	    return TitledWidget::title();
	}

	virtual QWidget *qwidget();
	virtual const QWidget *qwidget() const;
    };

    class ProgressIndicatorWidget : public TitledWidget, public Hui::ProgressIndicatorWidget {
	CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	Hui		&mHui;
	QProgressBar	*mProgressIndicator;
	string		mComment;
	float		mMin;
	float		mMax;

    public:
	ProgressIndicatorWidget(
		Hui &aHui,
		Pointer<GroupWidget> aParent,
		const string &aName,
		float aMin,
		float aMax,
		float aInitial,
		const string &aTitle,
		const string &aComment);

	virtual void set(float aValue);
	virtual float set() const;

	virtual void
	title(const string & aTitle)
	{
	    TitledWidget::title(aTitle);
	}

	virtual string
	title() const
	{
	    return TitledWidget::title();
	}

	virtual QWidget *qwidget();
	virtual const QWidget *qwidget() const;
    };

    class MenuItemWidget : public Widget, virtual public Hui::MenuItemWidget {
	CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	HuiQt		&mHui;
	MenuItemWidget	*mParent;
	bool		mDisabled;

    public:
	MenuItemWidget(
		Hui &aHui,
		const string &aName,
		Pointer<LanguageInterface::CallbackData> aCallbackData,
		MenuItemWidget *aParent);

	virtual void
	move(const Position &aPosition)
	{
	    Hui::MenuItemWidget::move(aPosition);
	}

	virtual Position
	position() const
	{
	    return Hui::MenuItemWidget::position();
	}

	virtual void
	resize(const Size &aSize)
	{
	    Hui::MenuItemWidget::resize(aSize);
	}

	virtual Size
	size() const
	{
	    return Hui::MenuItemWidget::size();
	}

	virtual Size
	minSize() const
	{
	    return Size();
	}

	virtual Size
	maxSize() const
	{
	    return Size();
	}

	virtual QWidget *
	qwidget()
	{
	    return 0;
	}

	virtual const QWidget *
	qwidget() const
	{
	    return 0;
	}

	virtual bool disabled() const;
	virtual void disable();
	virtual void enable();
	virtual void refresh();

	HuiQt &
	hui()
	{
	    return mHui;
	}

    protected:
	MenuItemWidget *
	parent()
	{
	    return mParent;
	}
    };

    class MenuSeparatorWidget : public MenuItemWidget, public Hui::MenuSeparatorWidget {
	CLASS_METHODS_DESCRIPTIONS_DECLARATION;

    public:
	MenuSeparatorWidget(
		Hui &aHui,
		const string &aName,
		MenuItemWidget *aParent);
    };

    class MenuButtonWidget : public MenuItemWidget, public Hui::MenuButtonWidget {
	CLASS_METHODS_DESCRIPTIONS_DECLARATION;

    public:
	MenuButtonWidget(
		Hui &aHui,
		const string &aName,
		const string &aText,
		Pointer<LanguageInterface::CallbackData> aCallbackData,
		const string &aKey,
		const string &aComment,
		MenuItemWidget *aParent);

	virtual void key(const string &aKey);
	virtual void text(const string &aText);

	virtual void click() { Hui::MenuButtonWidget::click(); }
    };

    class MenuSubmenuWidget : public MenuItemWidget, public Hui::MenuSubmenuWidget {
	CLASS_METHODS_DESCRIPTIONS_DECLARATION;

    public:
	MenuSubmenuWidget(
		Hui &aHui,
		const string &aName,
		const string &aText,
		const string &aComment,
		MenuItemWidget *aParent);

	virtual MenuItemWidget_P
	addButton(
		const string &aName,
		const string &aText,
		Pointer<LanguageInterface::CallbackData> aCallbackData,
		const string &aKey,
		const string &aComment,
		MenuItemWidget_P aBefore);

	virtual MenuItemWidget_P
	addSeparator(
		const string &aName,
		MenuItemWidget_P aBefore);

	virtual MenuItemWidget_P
	addSubmenu(
		const string &aName,
		const string &aText,
		const string &aComment,
		MenuItemWidget_P aBefore);

	virtual void
	add(
		MenuItemWidget_P aBefore,
		MenuItemWidget_P aItem);

	virtual void remove(MenuItemWidget_P aItem);
	virtual void text(const string &aText);
    };

    class MenuBarWidget : public MenuSubmenuWidget {
	CLASS_METHODS_DESCRIPTIONS_DECLARATION;

    public:
	MenuBarWidget(Hui &aHui);

	MenuItemWidget_P
	addSubmenu(
		const string &aName,
		const string &aText,
		const string &aComment,
		MenuItemWidget_P aBefore);

	void refresh();
    };

    const Subwidgets &
    subwidgets() const
    {
	return mWidgets;
    }

    Subwidgets &
    subwidgets()
    {
	return mWidgets;
    }

    static string translateAccel(const string &aText);

    /*
     * Variables
     */
    Subwidgets			mWidgets;
    Pointer<MenuSubmenuWidget>	mMenuBar;

public:
    /**
     * Constructor.
     */
    HuiQt(int argc, char **argv);

    /**
     * Run the application.
     */
    void run();

    /**
     * Create a widget by name.
     */
    Widget_P
    byName(const string &aName);

    /**
     * Create a window.
     */
    WindowWidget_P
    window(const string &aName,
	   const string &aTitle,
	   const Position &aPosition,
	   const Size &aSize,
	   const string &aAlignment,
	   bool no_menu = false);

    /**
     * Create a dialog.
     */
    DialogWidget_P
    dialog(Widget_P aParent,
	   const string &aName,
	   const string &aTitle,
	   const Position &aPosition,
	   const Size &aSize,
	   const string &aAlignment);

    /**
     * Create a menu bar.
     */
    MenuSubmenuWidget_P
    menuBar();

    /**
     */
    void processPendingEvents();

    /**
     * Returns an empty string if user pressed cancel button.
     */
    string
    fileSelectionDialog(const string &aTitle, const string &aWildCard);

    /**
     * Returns type of UI.
     */
    const char *type() const { return "graphics"; }

    /**
     * Returns size of UI.
     */
    Size size() const;

    /**
     * Exit the application.
     */
    void exit(int status);

    /**
     * Destructor.
     */
    ~HuiQt();
};

//======================================================================

#ifndef QDIALOG_H
#include <qdialog.h>
#endif

#ifndef QPUSHBT_H
#include <qpushbt.h>
#endif

#ifndef QLINED_H
#include <qlined.h>
#endif

#ifndef QCHKBOX_H
#include <qchkbox.h>
#endif

#ifndef QRADIOBT_H
#include <qradiobt.h>
#endif

#ifndef QLISTVIEW_H
#include <qlistview.h>
#endif

#ifndef QMLINED_H
#include <qmlined.h>
#endif

#ifndef QMAINWINDOW_H
#include <qmainwindow.h>
#endif

//======================================================================

class QResizableDialog : public QDialog
{
	Q_OBJECT;

public:
	QResizableDialog(QWidget *aParent,
			 const string &aName, bool aModal);
	//virtual ~QResizableDialog() { cerr << "~QResizableDialog()" << endl; }

	virtual void endModal() { accept(); }

signals:
	void resized(const QSize &aSize);
	void enteredLeft(bool aEntered);

protected:
	virtual void resizeEvent(QResizeEvent *event)
	{
		emit resized(event->size());
	}

	virtual void enterEvent(QResizeEvent *event)
	{
		emit enteredLeft(true);
		QDialog::enterEvent(event);
	}

	virtual void leaveEvent(QResizeEvent *event)
	{
		emit enteredLeft(false);
		QDialog::leaveEvent(event);
	}

}; // class QResizableDialog

//======================================================================

class QResizableMainWindow : public QMainWindow
{
	Q_OBJECT;

public:
	QResizableMainWindow(const string &aName);
	//virtual ~QResizableMainWindow() { cerr << "~QResizableMainWindow()" << endl; }

	signals:
	void resized(const QSize &aSize);
	void enteredLeft(bool aEntered);

protected:
	virtual void resizeEvent(QResizeEvent *event)
	{
		emit resized(event->size());
	}

	virtual void enterEvent(QResizeEvent *event)
	{
		emit enteredLeft(true);
		QMainWindow::enterEvent(event);
	}

	virtual void leaveEvent(QResizeEvent *event)
	{
		emit enteredLeft(false);
		QMainWindow::leaveEvent(event);
	}

}; // class QResizableMainWindow

//======================================================================

class QPushButtonEnterLeave : public QPushButton
{
	Q_OBJECT;

public:
	QPushButtonEnterLeave(const char *text,
			      QWidget *parent,
			      const char *name)
		: QPushButton(text, parent, name) {}

protected:
	virtual void focusInEvent(QFocusEvent *e)
	{
		emit enter_signal();
		QPushButton::focusInEvent(e);
	}

	virtual void focusOutEvent(QFocusEvent *e)
	{
		emit leave_signal();
		QPushButton::focusOutEvent(e);
	}

signals:
	void enter_signal();
	void leave_signal();

}; // class QPushButtonEnterLeave

//======================================================================

class QLineEditEnterLeave : public QLineEdit
{
	Q_OBJECT;

public:
	QLineEditEnterLeave(QWidget *parent,
			    const char *name)
		: QLineEdit(parent, name) {}

protected:
	virtual void focusInEvent(QFocusEvent *e)
	{
		emit enter_signal();
		QLineEdit::focusInEvent(e);
	}

	virtual void focusOutEvent(QFocusEvent *e)
	{
		emit leave_signal();
		QLineEdit::focusOutEvent(e);
	}

signals:
	void enter_signal();
	void leave_signal();

}; // class QLineEditEnterLeave

//======================================================================

class QCheckBoxEnterLeave : public QCheckBox
{
	Q_OBJECT;

public:
	QCheckBoxEnterLeave(const char *title,
			    QWidget *parent,
			    const char *name = 0)
		: QCheckBox(title, parent, name) {}

protected:
	virtual void focusInEvent(QFocusEvent *e)
	{
		emit enter_signal();
		QCheckBox::focusInEvent(e);
	}

	virtual void focusOutEvent(QFocusEvent *e)
	{
		emit leave_signal();
		QCheckBox::focusOutEvent(e);
	}

signals:
	void enter_signal();
	void leave_signal();

}; // class QCheckBoxEnterLeave

//======================================================================

class QRadioButtonEnterLeave : public QRadioButton
{
	Q_OBJECT;

public:
	QRadioButtonEnterLeave(const char *title,
			       QWidget *parent,
			       const char *name = 0)
		: QRadioButton(title, parent, name) {}

protected:
	virtual void focusInEvent(QFocusEvent *e)
	{
		emit enter_signal();
		QRadioButton::focusInEvent(e);
	}

	virtual void focusOutEvent(QFocusEvent *e)
	{
		emit leave_signal();
		QRadioButton::focusOutEvent(e);
	}

signals:
	void enter_signal();
	void leave_signal();

}; // class QRadioButtonEnterLeave

//======================================================================

class QListViewEnterLeave : public QListView
{
	Q_OBJECT;

public:
	QListViewEnterLeave(QWidget *parent,
			   const char *name)
		: QListView(parent, name) {}

protected:
	virtual void focusInEvent(QFocusEvent *e)
	{
		emit enter_signal();
		QListView::focusInEvent(e);
	}

	virtual void focusOutEvent(QFocusEvent *e)
	{
		emit leave_signal();
		QListView::focusOutEvent(e);
	}

signals:
	void enter_signal();
	void leave_signal();

}; // class QListBoxEnterLeave

//======================================================================

class QMultiLineEditEnterLeave : public QMultiLineEdit
{
	Q_OBJECT;

public:
	QMultiLineEditEnterLeave(QWidget *parent,
				 const char *name)
		: QMultiLineEdit(parent, name) {}

protected:
	virtual void focusInEvent(QFocusEvent *e)
	{
		emit enter_signal();
		QMultiLineEdit::focusInEvent(e);
	}

	virtual void focusOutEvent(QFocusEvent *e)
	{
		emit leave_signal();
		QMultiLineEdit::focusOutEvent(e);
	}

signals:
	void enter_signal();
	void leave_signal();

}; // class QMultiLineEditEnterLeave

//======================================================================

#endif // HuiQt_hh
