/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/include/hui/ProgressIndicatorHui.hh,v 1.8 2002/05/13 16:29:41 okumoto Exp $
 */

// -*- C++ -*-  Sun Jan  3 1999
// Eugene Skepner <eu@qub.com>
// $Id: ProgressIndicatorHui.hh,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef ProgressIndicatorHui_hh
#define ProgressIndicatorHui_hh

#include "ProgressIndicator.hh"
#include "Hui.hh"

//======================================================================

class ProgressIndicatorHui : public ProgressIndicator
{
  public:
     ProgressIndicatorHui(const string& aName, Hui::Hui_P aHui)
          : ProgressIndicator(aName),
            mWindow(aHui->window(aName + "_dialog", "Progress", Position(5, 9), Size(70, 6), "free", true /* no menu */)),
            mIndicator(mWindow->progressIndicator("indicator", 0, 1, 0, aName, Position(2, 1), ""))
          {
               mIndicator->resize(Size(64, mIndicator->size().height()));
               mWindow->show();
          }

     virtual ~ProgressIndicatorHui()
          {
               mWindow->hide();
          }

     virtual void progress(float aProgress)
          {
               mIndicator->set(aProgress);
          }

  private:
     Hui::GroupWidget_P			mWindow;
     Hui::ProgressIndicatorWidget_P	mIndicator;

}; // class ProgressIndicatorHui

//======================================================================

class ProgressIndicatorHuiStack : public ProgressIndicatorStack
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     ProgressIndicatorHuiStack(Hui::Hui_P aHui) : mHui(aHui) {}

     static Pointer<ProgressIndicatorStack> make(Hui::Hui_P aHui) { return new ProgressIndicatorHuiStack(aHui); }

  protected:
     virtual Pointer<ProgressIndicator> newProgressIndicator(const string& aName) { return new ProgressIndicatorHui(aName, mHui); }

  private:
     Hui::Hui_P mHui;

}; // class ProgressIndicatorHuiStack

//======================================================================

#endif // ProgressIndicatorHui_hh
