/*-
 * Copyright (c) 2002 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $FreeBSD: libh/include/part/fstab_file.hh,v 1.4 2002/04/22 18:53:54 antoine Exp $
 */

#ifndef FSTAB_FILE_HH
#define FSTAB_FILE_HH

extern "C" {
#include <fstab.h>
}

#include <vector>

#include "LanguageInterface.hh"

/**
 * a class to represent classic fstab structs
 *
 * @note unfortunatly, I couldn't design something inheriting directly
 * from struct fstab because I had problems getting something which
 * would compile and run properly. If anyone can get this to work, it
 * might be good since it would encapsulate the fstab struct properly.
 */
class cfstab
{

public:
     /// simple empty constructor
     cfstab() {}
     /// copy constructor from the classic fstab
     cfstab(fstab &f);
     /// empty descructor
     ~cfstab() {};

     /**
      * this is a string representation of this line
      *
      * @return a string conforming fstab(5)'s format
      */
     const string str();

     /**
      * mount the filesystem associated with this tab
      *
      * @throw Error mount(2) call failed
      * @note does not respect options yet
      */
     void    mount();

     string  fs_spec;		/** block special device name */
     string  fs_file;		/** filesystem path prefix */
     string  fs_vfstype;	/** File system type, ufs, nfs */
     string  fs_mntops;		/** Mount options ala -o */
     string  fs_type;		/** FSTAB_* from fs_mntops */
     int     fs_freq;		/** dump frequency, in days */
     int     fs_passno;		/** pass number on parallel fsck */

};

// ostream& operator<< (ostream&, const cfstab&);

/**
 * Representation of a /etc/fstab file
 *
 * Upon creation this class will read and initialize its vector of
 * cfstabs.
 *
 * Users of this class should manipulate the cfstabs vector directly
 * to add, sort, remove items to the fstab.
 *
 * @note there is no write() function yet
 */
class fstab_file
{

public:
     /// Constructor reading the /etc/fstab file
     fstab_file();
     /// close the file
     ~fstab_file() {
	  ::endfsent();
     }

     /// the vector of cfstabs representing all lines of the file
     vector<cfstab> cfstabs;

};

#endif
