#ifndef Hui_hh_8f9345bf
#define Hui_hh_8f9345bf
/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/Hui/Hui.hh,v 1.43 2002/08/21 10:52:06 okumoto Exp $
 */

#include <stdexcept>
#include <string>

extern "C" {
	#include "assert.h"
}

/*
 * Forward declarations.
 */
class HuiDialog;
class HuiMenuSubmenu;
class HuiToplevel;
class HuiWidget;
class HuiWindow;
class HuiRegion;

/**
 * Represent a libh user interface (abstract base class).
 */
class Hui {
protected:
	class Error : public runtime_error {
	public:
		Error(const string &what_arg)
		    : runtime_error(what_arg)
		{
		}
	};

public:
	/**
	 * Destructor.  @note Virtual because this object is a base class.
	 */
	virtual ~Hui();

	/**
	 * Create a window.
	 *
	 * @param aTitle	Text of the title
	 * @param aAlignment	How to arrange the objects in the window.
	 *			ARRANGE_FREE, ARRANGE_HORIZONTAL or
	 *			ARRANGE_VERTICAL
	 * @param region	Location and size of window
	 *
	 * @note tcl visible interface.
	 */
	virtual HuiWindow *
	window(
		const string &aTitle,
		const string &aAlignment,
		const HuiRegion &region) = 0;

	/**
	 * Create a dialog.
	 *
	 * @param aParent	???
	 * @param aTitle	Text of the title
	 * @param aAlignment	How to arrange the objects in the window.
	 *			ARRANGE_FREE, ARRANGE_HORIZONTAL or
	 *			ARRANGE_VERTICAL
	 * @param region	Location and size of window
	 *
	 * @note tcl visible interface.
	 */
	virtual HuiDialog *
	dialog(
		HuiWidget *aParent,
		const string &aTitle,
		const string &aAlignment,
		const HuiRegion &region) = 0;

	/**
	 * Returns an empty string if user pressed cancel button.
	 *
	 * @note tcl visible interface.
	 */
	virtual string
	fileSelectionDialog(const string &title, const string &patt) = 0;

	/**
	 * Remove widget.
	 *
	 * @note tcl visible interface.
	 */
	virtual void
	remove(HuiToplevel *widget) = 0;

	/**
	 * Run the application.
	 *
	 * @note tcl visible interface.
	 */
	virtual void run() = 0;

	/**
	 * Exit the application.
	 *
	 * @note tcl visible interface.
	 */
	virtual void exit(int status) = 0;
};

#endif /* Hui_hh_8f9345bf */
