/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/Hui/HuiCallback.cc,v 1.2 2002/08/18 08:12:56 okumoto Exp $
 */

#include "Hui/HuiCallback.hh"

#include <tcl.h>

/**
 * Copy constructor.
 */
HuiCallback::HuiCallback(const HuiCallback &rhs)
    : interp(rhs.interp),
      script(rhs.script)
{
	Tcl_IncrRefCount(script);
}

/**
 * Assignment operator.
 */
HuiCallback &
HuiCallback::operator=(const HuiCallback &rhs)
{
	set(rhs.interp, rhs.script);
	return *this;
}

/**
 * Destroy a Callback object.
 */
HuiCallback::~HuiCallback()
{
	if (script != NULL) {
		Tcl_DecrRefCount(script);
	}
}

/**
 * Set fields.
 */
void
HuiCallback::set(Tcl_Interp *aInterp, Tcl_Obj *aScript)
{
	interp = aInterp;
	if (script != NULL) {
		Tcl_DecrRefCount(script);
	}
	script = aScript;
	Tcl_IncrRefCount(script);
}

/**
 * Invoke call back script.
 */
void
HuiCallback::invoke() const
{
	assert(interp);
	assert(script);

	if (script != NULL) {
		Tcl_EvalObjEx(interp, script, 0);
	}
}

