#ifndef HuiGroup_hh_623a4a49
#define HuiGroup_hh_623a4a49
/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/Hui/HuiGroup.hh,v 1.5 2002/08/17 21:59:01 okumoto Exp $
 */

#include "Hui/HuiWidget.hh"

#include <string>
#include <vector>

class HuiButton;
class HuiCallback;
class HuiCheckBoxes;
class HuiContainer;
class HuiEmbedded;
class HuiInputLine;
class HuiLabel;
class HuiListBox;
class HuiProgressIndicator;
class HuiRadioMenu;
class HuiStringList;
class HuiTextEdit;

/**
 * Base class for top level widgets and containers.
 */
class HuiGroup : virtual public HuiWidget {

public:
	/**
	 *
	 */
	enum Arrangement {
		ARRANGE_FREE,
		ARRANGE_HORIZONTAL,
		ARRANGE_VERTICAL,
	};

	~HuiGroup();

	/**
	 * Create and add a button to the group.
	 */
	virtual HuiButton *
	button(
		const string &aText,
		const string &aButtonType,
		const HuiCallback &cb,
		const HuiRegion &region) = 0;

	/**
	 * Create and add a container to the group.
	 */
	virtual HuiContainer *
	container(
		const Position &aPosition,
		const string &aAlignment) = 0;

	/**
	 * Create and add a input line to the group.
	 */
	virtual HuiInputLine *
	inputLine(
		const string &aInitialText,
		int aMaxLength,
		const string &aValidator,
		const string &aTitle,
		const Position &aPosition) = 0;

	/**
	 * Create and add a label to the group.
	 */
	virtual HuiLabel *
	label(
		const string &aText,
		const HuiRegion &region) = 0;

	/**
	 * Create and add a list box to the group.
	 */
	virtual HuiListBox *
	listBox(
		const HuiStringList &items,
		const int aInitiallySelected,
		const string &aTitle,
		const HuiRegion &region) = 0;

	/**
	 * Create and add a progress indicator to the group.
	 */
	virtual HuiProgressIndicator *
	progressIndicator(
		float aMin,
		float aMax,
		float aInitial,
		const string &aTitle,
		const Position &aPosition) = 0;

	/**
	 * Create and add a text edit to the group.
	 */
	virtual HuiTextEdit *
	textEdit(
		const string &aInitialText,
		bool aAllowEdit,
		const string &aTitle,
		const HuiRegion &region) = 0;

	/**
	 * Create and add a checkbox to the group.
	 */
	virtual HuiCheckBoxes *
	checkBoxes(
		const vector<string> &aTexts,
		const vector<bool> &aInitialValues,
		const string &aTitle,
		const Position &aPosition,
		const vector<string> &labels) = 0;

	/**
	 * Create and add a radio button to the group.
	 */
	virtual HuiRadioMenu *
	radioMenu(
		const vector<string> &aTexts,
		const int aInitialValue,
		const string &aTitle,
		const Position &aPosition,
		const vector<string> &labels) = 0;

	/**
	 * Remove all objects from the group.
	 */
	virtual void erase() = 0;

	virtual void arrange() = 0;
	virtual void arrange(const string &aAlignment, bool arrange_new) = 0;

	virtual HuiWidget *current() const = 0;
	virtual void current(HuiEmbedded *Huia) = 0;
};

#endif /* HuiGroup_hh_623a4a49 */
