/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiQt/HuiQt.cc,v 1.6 2002/08/26 11:05:31 okumoto Exp $
 */

#include "HuiQt/HuiQt.hh"
#include "HuiQt/HuiQtApplication.hh"
#include "HuiQt/HuiQtDialog.hh"
#include "HuiQt/HuiQtWidget.hh"
#include "HuiQt/HuiQtWindow.hh"

/**
 * Constructor.
 */
HuiQt::HuiQt(int argc, char *argv[])
{
	app_info = new HuiQtApplication(argc, argv);
}

/**
 * Destructor.
 */
HuiQt::~HuiQt()
{
	delete app_info;
}

/**
 * Run the application.
 */
void
HuiQt::run()
{
	app_info->exec();
}

/**
 * Create a window.
 *
 * @param aTitle	name of window
 * @param aAlignment	how components are laid out in window
 * @param region	bounderies of window
 */
HuiWindow *
HuiQt::window(
	const string &aTitle,
	const string &aAlignment,
	const HuiRegion &region)
{
	HuiQtWindow *window =
		new HuiQtWindow(
			aTitle,
			aAlignment,
			region.bigger(8));

        topWidgets.push_back(window);

	return window;
}

/**
 * Create a dialog.
 */
HuiDialog *
HuiQt::dialog(
	HuiWidget *aParent,
	const string &aTitle,
	const string &aAlignment,
	const HuiRegion &region)
{
#if 0
	HuiQtDialog *dialog =
		new HuiQtDialog(
			aTitle,
			aAlignment,
			region);

        topWidgets.push_back(dialog);

	return dialog;
#else
	return NULL;
#endif
}

#if 0
void
HuiQt::exec(Hui *w)
{
	HuiQtWidget *dialog = dynamic_cast<HuiQtWidget *>(w);

        topWidgets.push_back(dialog);
	app_info->deskTop->execView(dialog->widget());
        topWidgets.remove(dialog);
}
#endif

/**
 * Returns an empty string if user pressed cancel button.
 *
 * @param aTitle	Title to put at top of dialog box
 * @param aWildCard
 */
string
HuiQt::fileSelectionDialog(const string &aTitle, const string &aWildCard)
{
	assert(0);
	return "filename";
}

/**
 *
 */
void
HuiQt::remove(HuiToplevel *toplevel)
{
	HuiQtToplevel *top =
		dynamic_cast<HuiQtToplevel *>(toplevel);

        topWidgets.remove(top);
}

/**
 * Exit the application.
 */
void
HuiQt::exit(int status)
{
	app_info->exit(status);
}

