/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiQt/HuiQtCheckBoxes.cc,v 1.1 2002/08/21 10:52:07 okumoto Exp $
 */

#include "HuiQt/HuiQtCheckBoxes.hh"

#define Uses_TStringCollection
#define Uses_TCheckBoxes
#define Uses_TRect

#include <tvision/tv.h>

class HuiTCheckBoxes : public TCheckBoxes {
	HuiQtCheckBoxes &mCheckBoxes;

public:
	HuiTCheckBoxes(
		HuiQtCheckBoxes &aCheckBoxes,
		const vector<string> &aTexts)
	    : TCheckBoxes(TRect(1, 1, 11, aTexts.size()),
			  HuiQtButtonsGroup::strings2tsitem(aTexts)),
	      mCheckBoxes(aCheckBoxes)
	{
	}

	bool
	state(int aItem) const
	{
		return const_cast<HuiTCheckBoxes*>(this)->mark(aItem);
	}

	void toggle(int aItem)
	{
		press(aItem);
	}

	void state(int aItem, bool aNewState)
	{
		if (mark(aItem) != aNewState)
			press(aItem);
	}

	void
	press(int aItem)
	{
		TCheckBoxes::press(aItem);
		assert(0);
		//mCheckBoxes.click(aItem);
	}

	string
	text(int aItem) const
	{
		if (strings && aItem >= 0 && aItem < strings->getCount())
			return (char*)strings->at(aItem);
		else
			return "";
	}

	void
	text(int aItem, const string &aNewText)
	{
		if (strings && aItem >= 0 && aItem < strings->getCount()) {
			delete strings->at(aItem);
			strings->atPut(aItem, const_cast<char*>(aNewText.c_str()));
		}
	}

	int
	size() const
	{
		if (strings)
			return strings->getCount();
		else
			return 0;
	}

	bool
	item_disabled(int boxno) const
	{
		return enableMask & (1L << boxno);
	}
};

/**
 * Constructor.
 */
HuiQtCheckBoxes::HuiQtCheckBoxes(
	HuiWidget *aParent,
	const vector<string> &aTexts,
	const vector<bool> &aInitialValues,
	const string &aTitle,
	const vector<string> &aSVector)
    : HuiQtButtonsGroup(aParent, aTitle, aSVector)
{
	mView = new HuiTCheckBoxes(*this, aTexts);

	for (vector<bool>::const_iterator i = aInitialValues.begin();
	     i != aInitialValues.end();
	     ++i)
		state(i - aInitialValues.begin(), *i);

	size_t maxTextLength = 0;
	for (int i = 0; i < numberOfItems(); ++i) {
		if (maxTextLength < HuiQtEmbedded::accel_text_length(text(i)))
		    maxTextLength = HuiQtEmbedded::accel_text_length(text(i));
	}
		
	resize(Size(maxTextLength + 6, numberOfItems() + (ttitle() ? 1 : 0)));
}

/**
 * Destructor.
 */
HuiQtCheckBoxes::~HuiQtCheckBoxes()
{
	delete mView;
}

bool
HuiQtCheckBoxes::state(int boxno, int aNewState)
{
	HuiTCheckBoxes *widget = dynamic_cast<HuiTCheckBoxes *>(mView);
	if (boxno >= 0 && boxno < numberOfItems()) {
		if (aNewState == -1967) {
			return widget->state(boxno);
		} else {
			const bool oldState = widget->state(boxno);
			widget->state(boxno, aNewState);
			return oldState;
		}
	} else {
		return false;
	}
}

bool
HuiQtCheckBoxes::state(int boxno) const
{
	HuiTCheckBoxes *widget = dynamic_cast<HuiTCheckBoxes *>(mView);
	if (boxno >= 0 && boxno < numberOfItems())
		return widget->state(boxno);
	else
		return false;
}

bool
HuiQtCheckBoxes::toggle(int boxno)
{
	HuiTCheckBoxes *widget = dynamic_cast<HuiTCheckBoxes *>(mView);
	if (boxno >= 0 && boxno < numberOfItems())
		widget->toggle(boxno);
	return state(boxno);
}

void
HuiQtCheckBoxes::text(int boxno, const string &aText)
{
	HuiTCheckBoxes *widget = dynamic_cast<HuiTCheckBoxes *>(mView);
	if (boxno >= 0 && boxno < numberOfItems())
		widget->text(boxno, aText);
}

string
HuiQtCheckBoxes::text(int boxno) const
{
	HuiTCheckBoxes *widget = dynamic_cast<HuiTCheckBoxes *>(mView);
	return widget->text(boxno);
}

int
HuiQtCheckBoxes::numberOfItems() const
{
	HuiTCheckBoxes *widget = dynamic_cast<HuiTCheckBoxes *>(mView);
	return widget->size();
}

bool
HuiQtCheckBoxes::item_disabled(int boxno) const
{
	HuiTCheckBoxes *widget = dynamic_cast<HuiTCheckBoxes *>(mView);
	return widget->item_disabled(boxno);
}

