/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiQt/HuiQtContainer.cc,v 1.1 2002/08/21 10:52:07 okumoto Exp $
 */

#include "HuiQt/HuiQtContainer.hh"

#define Uses_TGroup

#include <tvision/tv.h>

HuiQtContainer::HuiQtContainer(
	HuiWidget *aParent,
	const Position &aPosition,
	const Size &aSize)
    : HuiQtGroup(aParent)
{
	mView = new TGroup(TRect(
			aPosition.x,
			aPosition.y,
			aPosition.x + aSize.width,
			aPosition.y + aSize.height));
}

void
HuiQtContainer::resize(const Size &aSize, bool is_arranging = false)
{
	Dimension	w = aSize.width;
	Dimension	h = aSize.height;

	if (w == -1 && h == -1)
		return;

	if (w >= 0 && h >= 0) {
		arrange();
		return;
	}

	if (!is_arranging && (w >= 0 || h >= 0)) {
		mFixedSize = (w < 0) ? h : w;

		/* arrange parent again since we changed the fixed size */
		HuiQtContainer *cp =
			dynamic_cast<HuiQtContainer *>(mParent);
		HuiQtGroup *gp =
			dynamic_cast<HuiQtGroup *>(mParent);
		if (cp != NULL)
			cp->arrange();
		if (gp != NULL)
			gp->arrange();
		return;
	}
}

