#ifndef HuiQtEmbedded_hh_ee8225ca
#define HuiQtEmbedded_hh_ee8225ca
/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiQt/HuiQtEmbedded.hh,v 1.4 2002/08/25 07:37:57 okumoto Exp $
 */

#include "Hui/HuiEmbedded.hh"
#include "HuiQt/HuiQtWidget.hh"

#include <algorithm>

class TParamText;

/**
 * Base for all TVision objects that can be viewed.
 */
class HuiQtEmbedded : virtual public HuiEmbedded, public HuiQtWidget {
protected:
	TParamText	*mTitle;

public:
	HuiQtEmbedded(
		HuiWidget *aParent);

	~HuiQtEmbedded();

	void
	title(const string &aTitle);

	string
	title() const;

	void
	move(const Position &aPosition);

	void
	resize(const Size &aSize, bool is_arranging = false);

	Position
	position() const;

	Size
	size() const;

	TParamText *
	ttitle() const
	{
		return mTitle;
	}

	/**
	 * Accelerators into required format
	 */
	static string
	translateAccel(const string &aText)
	{
		string result = aText;
		string::size_type p = result.find('&');
		if (p != string::npos && result.length() > p + 1) {
			result.replace(p, 1, 1, '~');
			result.insert(p + 2, 1, '~');
		}
		return result;
	}


	static inline size_t
	accel_text_length(const string &aText)
	{
		return aText.length() -
			count(aText.begin(), aText.end(), '~') -
			count(aText.begin(), aText.end(), '&');
	}
};

#endif /* HuiQtEmbedded_hh_ee8225ca */
