#ifndef HuiQtGroup_hh_62259f39
#define HuiQtGroup_hh_62259f39
/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiQt/HuiQtGroup.hh,v 1.3 2002/08/22 11:14:56 okumoto Exp $
 */

#include "Hui/HuiGroup.hh"
#include "HuiQt/HuiQtWidget.hh"

#include <list>
#include <string>

/*
 * Forward declarations
 */
class HuiStringList;

/**
 */
class HuiQtGroup : virtual public HuiGroup, public HuiQtWidget {
	typedef list<HuiQtWidget *>	WidgetList;

	Arrangement	mArrangement;

protected:
	WidgetList	mWidgets;

	/**
	 * Add 'aWidget' to group.
	 */
	void
	insert(HuiWidget *aWidget);

	/**
	 * Constructor.
	 */
	HuiQtGroup(HuiWidget *aParent)
	    : HuiQtWidget(aParent)
	{
	}

public:
	HuiButton *
	button(
		const string &aText,
		const string &aButtonType,
		const HuiCallback &cb,
		const HuiRegion &region);

	HuiCheckBoxes *
	checkBoxes(
		const vector<string> &aTexts,
		const vector<bool> &aInitialValues,
		const string &aTitle,
		const Position &aPosition,
		const vector<string> &aSVector);

	HuiContainer *
	container(
		const Position &aPosition,
		const string &aAlignment);

	HuiInputLine *
	inputLine(
		const string &aInitialText,
		int aMaxLength,
		const string &aValidator,
		const string &aTitle,
		const Position &aPosition);

	HuiLabel *
	label(
		const string &aText,
		const HuiRegion &region);

	HuiListBox *
	listBox(
		const HuiStringList &items,
		const int aInitiallySelected,
		const string &title,
		const HuiRegion &region);

	HuiRadioMenu *
	radioMenu(
		const vector<string> &aTexts,
		const int aInitialValue,
		const string &aTitle,
		const Position &aPosition,
		const vector<string> &aSVector);

	HuiTextEdit *
	textEdit(
		const string &aInitialText,
		bool aAllowEdit,
		const string &aTitle,
		const HuiRegion &region);

	HuiProgressIndicator *
	progressIndicator(
		float aMin,
		float aMax,
		float aInitial,
		const string &aTitle,
		const Position &aPosition);

	/**
	 * Remove all Widgets from the group.
	 */
	void erase();

	void arrange();
	void arrange(const string &aAlignment, bool arrange_new);

	void current(HuiEmbedded *aWidget);
	HuiWidget *current() const;
	void aboutToChangeCurrent(TView *old, TView *current);
};

#endif /* HuiQtGroup_hh_62259f39 */
