/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiQt/HuiQtLabel.cc,v 1.3 2002/08/26 11:05:31 okumoto Exp $
 */

#include "HuiQt/HuiQtLabel.hh"
#include "HuiQt/HuiQtWidget.hh"

#include <qlabel.h>

/**
 * Constructor.
 */
HuiQtLabel::HuiQtLabel(
	HuiWidget *aParent,
	const string &aText,
	const HuiRegion &region)
    : HuiQtEmbedded(aParent)
{
	HuiQtWidget *p = dynamic_cast<HuiQtWidget *>(aParent);
	mView = new QLabel(aText.c_str(), p->widget());
	mView->move(region.a.x, region.a.y);
	//mView->resize(region.width(), region.height());
	mView->show();
}

/**
 * Destructor.
 */
HuiQtLabel::~HuiQtLabel()
{
	delete mView;
}

/**
 */
void
HuiQtLabel::text(const string &text)
{
	QLabel *widget = dynamic_cast<QLabel *>(mView);

	widget->setText(text.c_str());
}

/**
 */
string
HuiQtLabel::text() const
{
	QLabel *widget = dynamic_cast<QLabel *>(mView);

	return widget->text().latin1();
}

