/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiQt/HuiQtMenuBar.cc,v 1.1 2002/08/21 10:52:07 okumoto Exp $
 */

#include "Hui/HuiCallback.hh"
#include "HuiQt/HuiQtMenuBar.hh"

#define Uses_TMenu
#define Uses_TEvent
#define Uses_TMenuBar
#define Uses_TMenuItem

#include <tvision/tv.h>

#include <vector>
#include <set>

/**
 *
 */
class MyTMenuBar : public TMenuBar {
	typedef vector<HuiCallback>	Table;
	typedef set<ushort>		FreeTable;

	/**
	 * In tvision/views.h (cmSysWakeup = 40) is the last allocated
	 * cmd.  So we choose 100 as a base.
	 */
	static const ushort	cmBase = 100;

	ushort			cmMax;
	Table			mTable;
	FreeTable		mFree;
public:
	/**
	 * Constructor.
	 */
	MyTMenuBar(const TRect &bounds, TMenu *aMenu)
	    : TMenuBar(bounds, aMenu),
	      cmMax(cmBase)
	{
	}

	ushort
	addCmd(const HuiCallback &cb)
	{
		if (mFree.empty()) {
			mTable.push_back(cb);
			return cmMax++;
		} else {
			FreeTable::iterator i = mFree.begin();
			ushort cm = *i;
			mTable[cm - cmBase] = cb;
			mFree.erase(i);
			return cm;
		}
	}

	void 
        handleEvent(TEvent &event)
	{
		if (event.what == evCommand) {
			ushort cm = event.message.command;

			if ((cmBase <= cm) && (cm < cmMax)) {
				mTable[cm - cmBase].invoke();
			}
		}

		TMenuBar::handleEvent(event);
	}

	void
	delCmd(ushort command)
	{
	}
};

/**
 * Add item to menu, and draw the resulting menu bar.  Also add it to
 * the widget list.
 *
 * @param item		Menu item that will be added to menu bar.
 */
void
HuiQtMenuBar::append(HuiQtMenuItem *item)
{
	/*
	 * If the menu is empty make it the first item and the
	 * default.  Otherwise append it to the list of menu items.
	 */
	if (mMenu->items == NULL) {
		mMenu->items = item->mwidget();
		mMenu->deflt = mMenu->items;
	} else {
		/* Append new submenu to end of list */
		for (TMenuItem *i = mMenu->items; i != NULL; i = i->next) {
			if (i->next == NULL) {
				i->append(item->mwidget());
				break;
			}
		}
	}

	mView->draw();

	wl.push_back(item);
}

/**
 * Constructs an empty TVision MenuBar.
 */
HuiQtMenuBar::HuiQtMenuBar(
	const TRect &bounds)
    : HuiQtMenuSubmenu(*this),
      mMenu(new TMenu())
{
	mView = new MyTMenuBar(bounds, mMenu);
}

/**
 * Destroys a TVision MenuBar.
 */
HuiQtMenuBar::~HuiQtMenuBar()
{
}

/**
 */
ushort
HuiQtMenuBar::addCmd(const HuiCallback &cb)
{
	MyTMenuBar	*widget = dynamic_cast<MyTMenuBar *>(mView);

	return widget->addCmd(cb);
}

/**
 */
void
HuiQtMenuBar::delCmd(ushort command)
{
	MyTMenuBar	*widget = dynamic_cast<MyTMenuBar *>(mView);

	widget->delCmd(command);
}

