/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiQt/HuiQtMenuItem.cc,v 1.1 2002/08/21 10:52:07 okumoto Exp $
 */

#include "HuiQt/HuiQtMenuItem.hh"
#include "HuiQt/HuiQtEmbedded.hh"

#define Uses_TKeys
#define Uses_TMenuItem

#include <tvision/tv.h>

ushort
HuiQtMenuItem::key2keyCode(const string &aKey)
{
	struct KeyCode {
		const char	* const key;
		ushort		keyCode;
	};

	if (aKey.empty()) {
		return 0;
	}

	static const KeyCode sKeyCode[] = {
		{ "ALT-0", kbAlt0 },
		{ "ALT-1", kbAlt1 },
		{ "ALT-2", kbAlt2 },
		{ "ALT-3", kbAlt3 },
		{ "ALT-4", kbAlt4 },
		{ "ALT-5", kbAlt5 },
		{ "ALT-6", kbAlt6 },
		{ "ALT-7", kbAlt7 },
		{ "ALT-8", kbAlt8 },
		{ "ALT-9", kbAlt9 },
		{ "ALT-A", kbAltA },
		{ "ALT-B", kbAltB },
		{ "ALT-C", kbAltC },
		{ "ALT-D", kbAltD },
		{ "ALT-E", kbAltE },
		{ "ALT-F", kbAltF },
		{ "ALT-G", kbAltG },
		{ "ALT-H", kbAltH },
		{ "ALT-I", kbAltI },
		{ "ALT-J", kbAltJ },
		{ "ALT-K", kbAltK },
		{ "ALT-L", kbAltL },
		{ "ALT-M", kbAltM },
		{ "ALT-N", kbAltN },
		{ "ALT-O", kbAltO },
		{ "ALT-P", kbAltP },
		{ "ALT-Q", kbAltQ },
		{ "ALT-R", kbAltR },
		{ "ALT-S", kbAltS },
		{ "ALT-T", kbAltT },
		{ "ALT-U", kbAltU },
		{ "ALT-V", kbAltV },
		{ "ALT-W", kbAltW },
		{ "ALT-X", kbAltX },
		{ "ALT-Y", kbAltY },
		{ "ALT-Z", kbAltZ },

		{ "CTRL-A", kbCtrlA },
		{ "CTRL-B", kbCtrlB },
		{ "CTRL-C", kbCtrlC },
		{ "CTRL-D", kbCtrlD },
		{ "CTRL-E", kbCtrlE },
		{ "CTRL-F", kbCtrlF },
		{ "CTRL-G", kbCtrlG },
		{ "CTRL-H", kbCtrlH },
		{ "CTRL-I", kbCtrlI },
		{ "CTRL-J", kbCtrlJ },
		{ "CTRL-K", kbCtrlK },
		{ "CTRL-L", kbCtrlL },
		{ "CTRL-M", kbCtrlM },
		{ "CTRL-N", kbCtrlN },
		{ "CTRL-O", kbCtrlO },
		{ "CTRL-P", kbCtrlP },
		{ "CTRL-Q", kbCtrlQ },
		{ "CTRL-R", kbCtrlR },
		{ "CTRL-S", kbCtrlS },
		{ "CTRL-T", kbCtrlT },
		{ "CTRL-U", kbCtrlU },
		{ "CTRL-V", kbCtrlV },
		{ "CTRL-W", kbCtrlW },
		{ "CTRL-X", kbCtrlX },
		{ "CTRL-Y", kbCtrlY },
		{ "CTRL-Z", kbCtrlZ },

		{ "F1", kbF1 },
		{ "F2", kbF2 },
		{ "F3", kbF3 },
		{ "F4", kbF4 },
		{ "F5", kbF5 },
		{ "F6", kbF6 },
		{ "F7", kbF7 },
		{ "F8", kbF8 },
		{ "F9", kbF9 },
		{ "F10", kbF10 },

		{ "SHIFT-F1", kbShiftF1 },
		{ "SHIFT-F2", kbShiftF2 },
		{ "SHIFT-F3", kbShiftF3 },
		{ "SHIFT-F4", kbShiftF4 },
		{ "SHIFT-F5", kbShiftF5 },
		{ "SHIFT-F6", kbShiftF6 },
		{ "SHIFT-F7", kbShiftF7 },
		{ "SHIFT-F8", kbShiftF8 },
		{ "SHIFT-F9", kbShiftF9 },
		{ "SHIFT-F10", kbShiftF10 },

		{ "CTRL-F1", kbCtrlF1 },
		{ "CTRL-F2", kbCtrlF2 },
		{ "CTRL-F3", kbCtrlF3 },
		{ "CTRL-F4", kbCtrlF4 },
		{ "CTRL-F5", kbCtrlF5 },
		{ "CTRL-F6", kbCtrlF6 },
		{ "CTRL-F7", kbCtrlF7 },
		{ "CTRL-F8", kbCtrlF8 },
		{ "CTRL-F9", kbCtrlF9 },
		{ "CTRL-F10", kbCtrlF10 },

		{ "ALT-F1", kbAltF1 },
		{ "ALT-F2", kbAltF2 },
		{ "ALT-F3", kbAltF3 },
		{ "ALT-F4", kbAltF4 },
		{ "ALT-F5", kbAltF5 },
		{ "ALT-F6", kbAltF6 },
		{ "ALT-F7", kbAltF7 },
		{ "ALT-F8", kbAltF8 },
		{ "ALT-F9", kbAltF9 },
		{ "ALT-F10", kbAltF10 },

		{ "ALT-BACKSPACE", kbAltBack },
		{ "ALT-EQUAL", kbAltEqual },
		{ "ALT-MINUS", kbAltMinus },
		{ "ALT-SPACE", kbAltSpace },
		{ "BACKSPACE", kbBack },
		{ "CTRL-BACKSPACE", kbCtrlBack },
		{ "CTRL-DEL", kbCtrlDel },
		{ "CTRL-END", kbCtrlEnd },
		{ "CTRL-ENTER", kbCtrlEnter },
		{ "CTRL-HOME", kbCtrlHome },
		{ "CTRL-INS", kbCtrlIns },
		{ "CTRL-LEFT", kbCtrlLeft },
		{ "CTRL-PGDN", kbCtrlPgDn },
		{ "CTRL-PGUP", kbCtrlPgUp },
		{ "CTRL-PRTSC", kbCtrlPrtSc },
		{ "CTRL-RIGHT", kbCtrlRight },
		{ "DEL", kbDel },
		{ "DOWN", kbDown },
		{ "END", kbEnd },
		{ "ENTER", kbEnter },
		{ "ESC", kbEsc },
		{ "GRAYMINUS", kbGrayMinus },
		{ "GRAYPLUS", kbGrayPlus },
		{ "HOME", kbHome },
		{ "INS", kbIns },
		{ "LEFT", kbLeft },
		{ "PGDN", kbPgDn },
		{ "PGUP", kbPgUp },
		{ "RIGHT", kbRight },
		{ "SHIFT-DEL", kbShiftDel },
		{ "SHIFT-INS", kbShiftIns },
		{ "SHIFT-TAB", kbShiftTab },
		{ "TAB", kbTab },
		{ "UP", kbUp },

		{ NULL, 0 }
	};

	for (const KeyCode *kc = sKeyCode; kc->key != NULL; ++kc) {
		if (strcasecmp(kc->key, aKey.c_str()) == 0) {
			return kc->keyCode;
		}
	}
	return tolower(aKey[0]);
}

HuiQtMenuItem::HuiQtMenuItem(
	HuiWidget *aParent)
    : HuiQtWidget(aParent),
      mMenuItem(NULL)
{
}

void
HuiQtMenuItem::disable()
{
	mMenuItem->disabled = True;
}


void
HuiQtMenuItem::enable()
{
	mMenuItem->disabled = False;
}

bool
HuiQtMenuItem::disabled() const
{
	return (mMenuItem->disabled == True);
}

void
HuiQtMenuItem::text(const string &aText)
{
	delete mMenuItem->name;
	mMenuItem->name =
		newStr(HuiQtEmbedded::translateAccel(aText).c_str());
	mText = aText;
}

