/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiQt/HuiQtRadioMenu.cc,v 1.1 2002/08/21 10:52:07 okumoto Exp $
 */

#include "HuiQt/HuiQtRadioMenu.hh"

#define Uses_TRadioButtons
#define Uses_TRect
#define Uses_TStringCollection

#include <tvision/tv.h>

class HuiTRadioMenu : public TRadioButtons {
	HuiQtRadioMenu &mRadioMenu;

public:
	HuiTRadioMenu(
		HuiQtRadioMenu &aRadioMenu,
		const vector<string> &aTexts)
	    : TRadioButtons(
			TRect(1, 1, 11, aTexts.size()),
			HuiQtButtonsGroup::strings2tsitem(aTexts)),
	      mRadioMenu(aRadioMenu)
	{
	}

	int
	set() const
	{
		ushort val;
		const_cast<HuiTRadioMenu*>(this)->getData(&val);
		return(val);
	}

	void
	toggle(int aItem)
	{
		press(aItem);
	}

	void
	set(int aItem)
	{
		if (!mark(aItem))
			press(aItem);
	}

	void
	press(int aItem)
	{
		TRadioButtons::press(aItem);
		assert(0);
		//mRadioMenu.click(aItem);
	}

	string
	text(int aItem) const
	{
		if (strings && aItem >= 0 && aItem < strings->getCount())
			return (char*)strings->at(aItem);
		else
			return string();
	}

	void
	text(int aItem, const string &aNewText)
	{
		if (strings && aItem >= 0 && aItem < strings->getCount()) {
			delete strings->at(aItem);
			strings->atPut(aItem, const_cast<char*>(aNewText.c_str()));
		}
	}

	int
	size() const
	{
		if (strings)
			return strings->getCount();
		else
			return 0;
	}

	bool
	item_disabled(int boxno) const
	{
		return enableMask & (1L << boxno);
	}
};

HuiQtRadioMenu::HuiQtRadioMenu(
	HuiWidget *aParent,
	const vector<string> &aTexts,
	int aInitialValue,
	const string &aTitle,
	const vector<string> &aSVector)
    : HuiQtButtonsGroup(aParent, aTitle, aSVector)
{
	mView = new HuiTRadioMenu(*this, aTexts);
#if 0
	if (ttitle())
		ttitle()->link(mView);
#else
	assert(0);
#endif

	set(aInitialValue);

	size_t maxTextLength = 0;
	for (int i = 0; i < numberOfItems(); ++i) {
		if (maxTextLength < HuiQtEmbedded::accel_text_length(text(i)))
			maxTextLength = HuiQtEmbedded::accel_text_length(text(i));
	}
	resize(Size(maxTextLength + 6, numberOfItems() + (ttitle() ? 1 : 0)));
}

HuiQtRadioMenu::~HuiQtRadioMenu()
{
	delete mView;
}

void
HuiQtRadioMenu::set(int itemno)
{
	HuiTRadioMenu	*widget = dynamic_cast<HuiTRadioMenu *>(mView);
	widget->set(itemno);
}

int
HuiQtRadioMenu::set() const
{
	HuiTRadioMenu	*widget = dynamic_cast<HuiTRadioMenu *>(mView);
	return widget->set();
}

void
HuiQtRadioMenu::text(int boxno, const string &aText)
{
	HuiTRadioMenu	*widget = dynamic_cast<HuiTRadioMenu *>(mView);
	if (boxno >= 0 && boxno < numberOfItems())
		widget->text(boxno, aText);
}


string
HuiQtRadioMenu::text(int boxno) const
{
	HuiTRadioMenu	*widget = dynamic_cast<HuiTRadioMenu *>(mView);
	return widget->text(boxno);
}

int
HuiQtRadioMenu::numberOfItems() const
{
	HuiTRadioMenu	*widget = dynamic_cast<HuiTRadioMenu *>(mView);
	return widget->size();
}

bool
HuiQtRadioMenu::item_disabled(int boxno) const
{
	HuiTRadioMenu	*widget = dynamic_cast<HuiTRadioMenu *>(mView);
	return widget->item_disabled(boxno);
}

