/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiQt/HuiQtTable.cc,v 1.1 2002/08/21 10:52:07 okumoto Exp $
 */

#include "Hui/HuiStringList.hh"
#include "HuiQt/HuiQtGroup.hh"
#include "HuiQt/HuiQtListBox.hh"

#define Uses_TEvent
#define Uses_TGroup
#define Uses_TKeys
#define Uses_TRect
#define Uses_TScrollBar
#include <tvision/tv.h>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

typedef vector<string>	SList;

class HuiTListBox : public TGroup {
	TScrollBar		hsb;
	TScrollBar		vsb;

	SList			mData;
	size_t			mFocused;
	size_t			mTopItem;
	HuiQtListBox		&owner;
public:
	/**
	 * Constructor.
	 */
	HuiTListBox(
		const HuiRegion &region,
		const HuiStringList &items,
		HuiQtListBox &aListBox)
	    : TGroup(TRect(
			region.a.x, region.a.y,
			region.b.x, region.b.y)),
	      hsb(TRect(
			0, region.height() - 1,
			region.width() - 1, region.height())),
	      vsb(TRect(
			region.width() - 1, 0,
			region.width(), region.height() - 1)),
	      owner(aListBox)
	{
		options |= ofFirstClick | ofSelectable;
		eventMask |= evBroadcast;

		insert(&hsb);
		insert(&vsb);
		hsb.show();
		vsb.show();

		hsb.setStep(size.x / 2, 1);
		vsb.setStep(size.y - 1, 1);
	}
#if 0
	void
	changeBounds(const TRect &bounds)
	{
		TView::changeBounds(TRect(
			bounds.a.x, bounds.a.y,
			bounds.b.x, bounds.b.y));

		hsb.setBounds(TRect(
			bounds.a.x + 1, bounds.b.y - 1,
			bounds.b.x, bounds.b.y));

		vsb.setBounds(TRect(
			bounds.a.x, bounds.a.y,
			bounds.a.x + 1, bounds.b.y - 1));

		hsb.setStep(size.x / 2, hsb.arStep);
		vsb.setStep(size.y, vsb.arStep);

		itemsChanged();
	}

	void
	draw()
	{
		ushort normalColor;
		ushort selectedColor;
		ushort focusedColor;

		if ((state & (sfSelected|sfActive)) == (sfSelected|sfActive)) {
			normalColor = getColor(1);
			focusedColor = getColor(3);
			selectedColor = getColor(4);
		} else {
			normalColor = getColor(2);
			focusedColor = 0;
			selectedColor = getColor(4);
		}

		const short indent = hsb.value;

		TDrawBuffer b;

		for (short i = 0; i < size.y - 1; i++) {
			const size_t item =  i + mTopItem;
			ushort color;
			uchar scOff;
			if ((state & (sfSelected | sfActive)) == (sfSelected | sfActive) && mFocused == item && item < mData.size()) {
				color = focusedColor;
				setCursor(0 + 1, i);
				scOff = 0;
			} else if (item == mFocused && item < mData.size()) {
				color = selectedColor;
				scOff = 2;
			} else {
				color = normalColor;
				scOff = 4;
			}

			b.moveChar(0, ' ', color, size.x + 1);
			if (item < mData.size()) {
				const string text = mData[item].substr(indent, size.x - 2);
				b.moveStr(1, text.c_str(), color);
				if (indent > 0)
					b.moveChar(0, '<', color, 1);
				if (mData[item].length() > size_t(indent + size.x - 2))
					b.moveChar(size.x - 1, '>', color, 1);
			} else if (i == 0) {
	#if 0
				b.moveStr(1, _("<empty>"), getColor(1));
	#else
				//assert(0);
	#endif
			}

			writeLine(1, i, size.x - 1, 1, b);
		}
	}

	TPalette &
	getPalette() const
	{
#define cpListViewer "\032\032\033\034\035"
		static TPalette palette(
				cpListViewer,
				sizeof(cpListViewer) - 1);
		return palette;
	}

	void
	handleEvent(TEvent &event)
	{
		TView::handleEvent(event);

		if (event.what == evMouseDown) {
			const int mouseAutosToSkip = 4;
			const ushort colWidth = size.x + 1;
			size_t oldItem = mFocused;
			TPoint mouse = makeLocal(event.mouse.where);
			size_t newItem;
			if (mouseInView(event.mouse.where)) {
				newItem = mouse.y + (size.y * (mouse.x / colWidth)) + mTopItem;
			}
			else
				newItem = oldItem;
			ushort count = 0;
			do {
				if (newItem != oldItem) {
					focus(newItem);
					drawView();
				}
				oldItem = newItem;
				mouse = makeLocal(event.mouse.where);
				if (mouseInView(event.mouse.where))
					newItem = mouse.y + (size.y * (mouse.x / colWidth)) + mTopItem;
				else {
					if (event.what == evMouseAuto)
						count++;
					if (count == mouseAutosToSkip) {
						count = 0;
						if (mouse.y < 0)
							newItem = mFocused - 1;
						else if (mouse.y >= size.y)
							newItem = mFocused + 1;
					}
				}
				if (event.mouse.eventFlags & meDoubleClick)
					break;
			} while (mouseEvent(event, evMouseMove | evMouseAuto));
			if (newItem != oldItem) {
				focus(newItem);
				drawView();
			}
			clearEvent(event);
		}
		else if (event.what == evKeyDown) {
			size_t newItem = mFocused;
			switch (ctrlToArrow(event.keyDown.keyCode)) {
			case kbUp:
				if (mFocused > 0)
					newItem = mFocused - 1;
				break;
			case kbDown:
				if (mFocused < mData.size() - 1)
					newItem = mFocused + 1;
				break;
			case kbRight:
				hsb.setValue(hsb.value + 1);
				break;
			case kbLeft:
				hsb.setValue(hsb.value - 1);
				break;
			case kbPgDn:
				newItem = mFocused + size.y - 2;
				if (newItem >= mData.size())
					newItem = mData.size() - 1;
				break;
			case  kbPgUp:
				if (mFocused > size_t(size.y))
					newItem = mFocused - size.y + 2;
				else
					newItem  = 0;
				break;
			case kbHome:
				newItem = 0;
				break;
			case kbEnd:
				newItem = mData.size() - 1;
				break;
			case kbCtrlPgDn:
				newItem = mTopItem + size.y - 1;
				if (newItem >= mData.size())
					newItem = mData.size() - 1;
				break;
			case kbCtrlPgUp:
				newItem = mTopItem;
				break;
			default:
				return;
			}
			focus(newItem);
			drawView();
			clearEvent(event);
		}
		else if (event.what == evBroadcast) {
			if ((options & ofSelectable) != 0) {
				if (event.message.command == cmScrollBarClicked &&
					(event.message.infoPtr == &hsb ||
					event.message.infoPtr == &vsb))
					focus();
				else if (event.message.command == cmScrollBarChanged) {
					if (&vsb == event.message.infoPtr) {
						focus(vsb.value);
						drawView();
					}
					else if (&hsb == event.message.infoPtr)
						drawView();
				}
			}
		}
	}

	void
	setState(ushort aState, Boolean enable)
	{
		TView::setState(aState, enable);
		if ((aState & (sfSelected | sfActive | sfVisible)) != 0) {
			if (getState(sfActive) && getState(sfVisible))
				hsb.show();
			else
				hsb.hide();
			if (getState(sfActive) && getState(sfVisible))
				vsb.show();
			else
				vsb.hide();
			drawView();
		}
	}

	void
	shutDown()
	{
		hsb.shutDown();
		vsb.shutDown();
	}

protected:


	const char *
	streamableName() const
	{
		return("HuiTListBox");
	}
#endif
public:
	void
	save_texts(const SList &aTexts, const string &aTitle)
	{
		owner.mTexts = aTexts;
		owner.mTitleText = aTitle;

		mData = columns_text_magic(
				owner.mTexts,
				owner.mTitleText);

		itemsChanged();

		owner.set_title(columns_titletext_magic(
				owner.mTexts,
				owner.mTitleText));
	}

	SList
	columns_text_magic(const SList &aTexts, const string &aTitle)
	{
		SList	newTexts = aTexts;
		string	newTitle = aTitle;

		string::size_type max;
		do {
			max = widest_column(newTexts, newTitle);
			if (max != string::npos) {
				/*
				 * Replace each \t with 2 at least one tab
				 * up to widest column
				 */
				string::size_type pos;
				for (size_t item = 0; item < newTexts.size(); ++item) {
					pos = newTexts[item].find("\t", 0);

					if (pos != newTexts[item].npos) {
						while (pos != max + 1)
							newTexts[item].insert(pos++, 1, ' ');
						newTexts[item].replace(pos, 1, ' ');
					}
				}
				pos = newTitle.find("\t", 0);
				if (pos != newTitle.npos) {
					while(pos != max + 1)
						newTitle.insert(pos++, 1, ' ');
					newTitle.replace(pos, 1, ' ');
				}
			}
		} while (max != string::npos);

		return(newTexts);
	}

	void
	itemsChanged()
	{
		if (mFocused >= mData.size())
			mFocused = 0;
		if (mTopItem >= mData.size())
			mTopItem = 0;

		if (mData.size() > 0)
			vsb.setParams(mFocused, 0, mData.size() > 0 ? mData.size() - 1 : 0, vsb.pgStep, vsb.arStep);

		size_t maxTextLength = 0;
		for (size_t i = 0; i < mData.size(); ++i)
			if (mData[i].length() > maxTextLength)
				maxTextLength = mData[i].length();
		int maxValue = int(maxTextLength) - int(size.x) + 2;
		if (maxValue < 0)
			maxValue = 0;
		hsb.setParams(hsb.value, 0, maxValue, hsb.pgStep, hsb.arStep);

		drawView();
	}

	string
	columns_titletext_magic(const SList &aTexts, const string &aTitle)
	{
		SList	newTexts = aTexts;
		string		newTitle = aTitle;

		string::size_type max;
		do {
			max = widest_column(newTexts, newTitle);
			if (max != string::npos) {
				/*
				 * Replace each \t with 2 at least one tab
				 * up to widest column
				 */
				string::size_type pos;
				for (size_t item = 0; item < newTexts.size(); item++) {
					pos = newTexts[item].find("\t", 0);

					if (pos != newTexts[item].npos) {
						while (pos != max + 1)
							newTexts[item].insert(pos++, 1, ' ');
						newTexts[item].replace(pos, 1, ' ');
					}
				}
				pos = newTitle.find("\t", 0);
				if (pos != newTitle.npos) {
					while (pos != max + 1)
						newTitle.insert(pos++, 1, ' ');
					newTitle.replace(pos, 1, ' ');
				}
			}
		} while (max != string::npos);

		return(newTitle);
	}

	string::size_type
	widest_column(
		const SList &aTexts,
		const string &aTitle)
	{
		string::size_type	max = 0;
		bool			found = 0;
		string::size_type	tmp;
		size_t			item;

		for (item = 0; item < aTexts.size(); item++) {
			tmp = aTexts[item].find("\t", 0);
			if (tmp != aTexts[item].npos) {
				max = MAX(max, tmp);
				found = 1;
			}
		}

		tmp = aTitle.find("\t", 0);
		if (tmp != aTitle.npos) {
			max = MAX(max, tmp);
			found = 1;
		}

		if (found)
			return max;
		else
			return string::npos;
	}

	void
	focus(size_t item)
	{
		if (item >= mData.size())
			item = mData.size() - 1;

		const bool changed = mFocused != item;

		mFocused = item;

		vsb.setValue(item);

		if (item < mTopItem)
			mTopItem = item;
		else if (item > mTopItem + size.y - 2 && size.y > 0)
			mTopItem = item - size.y + 2;

		if (changed)
			owner.highlighted(mFocused);
	}

	size_t
	focus() const
	{
		return mFocused;
	}

};

/* xxxxx */

/**
 * Constructor.
 */
HuiQtListBox::HuiQtListBox(
	HuiWidget *aParent,
	const string &aTitle,
	const HuiStringList &items,
	int aInitiallySelected,
	const HuiRegion &region)
    : HuiQtEmbedded(aParent)
{
	mView = new HuiTListBox(
			region,
			items,
			*this);

	HuiTListBox *widget = dynamic_cast<HuiTListBox *>(mView);
SList	xxx_items;
	widget->save_texts(xxx_items, aTitle);

	select(aInitiallySelected);
}

HuiQtListBox::~HuiQtListBox()
{
	delete mView;
}


int
HuiQtListBox::numberOfItems() const
{
	return mTexts.size();
}

string
HuiQtListBox::text(int aItemNo) const
{
	if (aItemNo >= numberOfItems())
#if 0
		throw runtime_error(string("ListBox itemno ")
				    + to_string(aItemNo)
				    + " is out of range 0.."
				    + to_string(numberOfItems() - 1));
#else
		assert(0);
#endif
	return mTexts[aItemNo];
}

void
HuiQtListBox::text(int aItemNo, const string &aText)
{
	if (aItemNo < 0) {
		return;
	}

	if (aItemNo >= numberOfItems()) {
#if 0
		throw runtime_error(string("ListBox itemno ")
				    + to_string(aItemNo)
				    + " is out of range 0.."
				    + to_string(numberOfItems() - 1));
#else
		assert(0);
#endif
	}

	mTexts[aItemNo] = aText;

	HuiTListBox *widget = dynamic_cast<HuiTListBox *>(mView);
	widget->save_texts(mTexts, mTitleText);
}

int
HuiQtListBox::select() const
{
	HuiTListBox *widget = dynamic_cast<HuiTListBox *>(mView);
	return widget->focus();
}

void
HuiQtListBox::select(int aItemNo)
{
	if (aItemNo >= numberOfItems()) {
#if 0
		throw runtime_error(string("ListBox itemno ")
			+ to_string(aItemNo)
			+ " is out of range 0.."
			+ to_string(numberOfItems() - 1));
#else
		assert(0);
#endif
	}

	HuiTListBox *widget = dynamic_cast<HuiTListBox *>(mView);
	widget->focus(aItemNo);
}

void
HuiQtListBox::add(int aBefore, const string &aText)
{
	mTexts.insert(mTexts.begin() + aBefore, aText);

	HuiTListBox *widget = dynamic_cast<HuiTListBox *>(mView);
	widget->save_texts(mTexts, mTitleText);
}

void
HuiQtListBox::remove(int aItemNo)
{
	if (aItemNo >= numberOfItems()) {
#if 0
	    throw runtime_error(string("ListBox itemno ")
				+ to_string(aItemNo)
				+ " is out of range 0.."
				+ to_string(numberOfItems() - 1));
#else
		assert(0);
#endif
	}

	mTexts.erase(mTexts.begin() + aItemNo);

	HuiTListBox *widget = dynamic_cast<HuiTListBox *>(mView);
	widget->save_texts(mTexts, mTitleText);
}

void
HuiQtListBox::title(const string &aTitle)
{
	HuiTListBox *widget = dynamic_cast<HuiTListBox *>(mView);
	widget->save_texts(mTexts, aTitle);
}

string
HuiQtListBox::title() const
{
	return mTitleText;
}

void
HuiQtListBox::set_title(const string &aTitle)
{
	HuiQtEmbedded::title(" " + aTitle);
}

void
HuiQtListBox::removeAll()
{
	mTexts.clear();

	HuiTListBox *widget = dynamic_cast<HuiTListBox *>(mView);
	widget->save_texts(mTexts, mTitleText);
}

void
HuiQtListBox::highlighted(int itemno)
{
}

