/*-
 * Copyright (c) 1998-2002 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiQt/HuiQtWidget.cc,v 1.2 2002/08/22 11:14:56 okumoto Exp $
 */

#include "HuiQt/HuiQtWidget.hh"
#include "HuiQt/HuiQtGroup.hh"

#include <qwidget.h>

/**
 * Constructor.
 *
 * @param parent	Pointer to widget containing this object,
 *			else NULL.
 */
HuiQtWidget::HuiQtWidget(HuiWidget *parent)
    : mParent(parent),
      mView(NULL),
      mFixedSize(-1)
{
}

/**
 * Move the widget to the new position.
 */
void
HuiQtWidget::move(const Position &pos)
{
	mView->move(pos.x, pos.y);
}

/**
 * Resize the widget.
 */
void
HuiQtWidget::resize(const Size &size, bool is_arranging = false)
{
	mView->move(size.width, size.height);
}

Position
HuiQtWidget::position() const
{
	return Position(mView->x(), mView->y());
}

Size
HuiQtWidget::size() const
{
	return Size(mView->width(), mView->height());
}

void
HuiQtWidget::show()
{
	mView->show();
}

void
HuiQtWidget::hide()
{
	mView->hide();
}

bool
HuiQtWidget::hidden() const
{
	return mView->isVisible();
}

void
HuiQtWidget::disable()
{
	mView->setEnabled(false);
}

void
HuiQtWidget::enable()
{
	mView->setEnabled(true);
}

bool
HuiQtWidget::disabled() const
{
	return (mView->isEnabled() == false);
}

