/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiQt/HuiQtWindow.cc,v 1.4 2002/08/25 07:37:57 okumoto Exp $
 */

#include "Hui/Dim.hh"
#include "HuiQt/HuiQt.hh"
#include "HuiQt/HuiQtMenuBar.hh"
#include "HuiQt/HuiQtWindow.hh"

#include <qmainwindow.h>

/**
 * Represent a window which can contain a menu bar.
 */
class MyQWindow : public QMainWindow {
	HuiQtMenuBar	*mMenuBar;

public:
	/**
	 * Construct a TVision window.  The window created is
	 * not numbered (wnNoNumber), so you can not use any of the
	 * ALlt-n keys to select this window.
	 *
	 * @param region	Location and Size of window
	 * @param aTitle	Title displayed for the window.
	 */
	MyQWindow(
		const HuiRegion &region,
		const string &aTitle)
	    : mMenuBar(NULL)
	{
		setCaption(aTitle.c_str());
		move(region.a.x, region.a.y);
		resize(region.width(), region.height());
	}

	/**
	 * Return a pointer to the menu bar.  If none exists, create
	 * one and return a pointer to it.
	 */
	HuiMenuSubmenu *
	getMenuBar()
	{
		if (mMenuBar == NULL) {
//			mMenuBar = new HuiQtMenuBar(menuBar());
		}
		return mMenuBar;
	}
};

/**
 * Construct a top level window.
 */
HuiQtWindow::HuiQtWindow(
	const string &aTitle,
	const string &aAlignment,
	const HuiRegion &region)
  : HuiQtToplevel(NULL)
{
	mView = new MyQWindow(
			region,
			aTitle);
	mView->show();
}

/**
 * Destructor.
 */
HuiQtWindow::~HuiQtWindow()
{
	delete mView;
}

/**
 * Return a pointer to the menu bar.
 */
HuiMenuSubmenu *
HuiQtWindow::menuBar()
{
	MyQWindow *window = dynamic_cast<MyQWindow *>(mView);

	return window->getMenuBar();
}

