/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiTv/HuiTv.cc,v 1.42 2002/08/20 10:41:42 okumoto Exp $
 */

#include "HuiTv/HuiTv.hh"
#include "HuiTv/HuiTvApplication.hh"
#include "HuiTv/HuiTvDialog.hh"
#include "HuiTv/HuiTvWidget.hh"
#include "HuiTv/HuiTvWindow.hh"

#define Uses_TFileDialog
#define Uses_TDeskTop

#include <tvision/tv.h>

/**
 * Constructor.
 */
HuiTv::HuiTv(int argc, char *argv[])
{
	app_info = new HuiTvApplication(argc, argv);
}

/**
 * Destructor.
 */
HuiTv::~HuiTv()
{
	destroy(app_info);
}

/**
 * Run the application.
 */
void
HuiTv::run()
{
	app_info->run();
}

/**
 * Create a window.
 *
 * @param aTitle	name of window
 * @param aAlignment	how components are laid out in window
 * @param region	bounderies of window
 */
HuiWindow *
HuiTv::window(
	const string &aTitle,
	const string &aAlignment,
	const HuiRegion &region)
{
	HuiTvWindow *window =
		new HuiTvWindow(
			aTitle,
			aAlignment,
			region);

	app_info->deskTop->insert(window->widget());
        topWidgets.push_back(window);

	return window;
}

/**
 * Create a dialog.
 */
HuiDialog *
HuiTv::dialog(
	HuiWidget *aParent,
	const string &aTitle,
	const string &aAlignment,
	const HuiRegion &region)
{
	HuiTvDialog *dialog =
		new HuiTvDialog(
			aTitle,
			aAlignment,
			region);

	app_info->deskTop->insert(dialog->widget());
        topWidgets.push_back(dialog);

	return dialog;
}

#if 0
void
HuiTv::exec(Hui *w)
{
	HuiTvWidget *dialog = dynamic_cast<HuiTvWidget *>(w);

        topWidgets.push_back(dialog);
	app_info->deskTop->execView(dialog->widget());
        topWidgets.remove(dialog);
}
#endif

/**
 * Returns an empty string if user pressed cancel button.
 *
 * @param aTitle	Title to put at top of dialog box
 * @param aWildCard
 */
string
HuiTv::fileSelectionDialog(const string &aTitle, const string &aWildCard)
{
	/* XXX Qt uses a different way of the initial path */
	TFileDialog* dialog =
		new TFileDialog(
			aWildCard.c_str(),
			aTitle.c_str(),
			"",
			fdOKButton,
			0);

	ushort status = app_info->deskTop->execView(dialog);
	char filename[1024] = "";

	if (status != cmCancel)
		dialog->getFileName(filename);
	delete dialog;

	return filename;
}

/**
 *
 */
void
HuiTv::remove(HuiToplevel *toplevel)
{
	HuiTvToplevel *top =
		dynamic_cast<HuiTvToplevel *>(toplevel);

        topWidgets.remove(top);
	app_info->deskTop->remove(top->widget());
}

/**
 * Exit the application.
 */
void
HuiTv::exit(int status)
{
	/* Send Quite message to TApplication */
	message(app_info, evCommand, cmQuit, NULL);
}

