/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiTv/HuiTvButton.cc,v 1.40 2002/08/20 10:41:42 okumoto Exp $
 */

#include "Hui/HuiCallback.hh"
#include "HuiTv/HuiTv.hh"
#include "HuiTv/HuiTvApplication.hh"
#include "HuiTv/HuiTvButton.hh"
#include "HuiTv/HuiTvContainer.hh"
#include "HuiTv/HuiTvDialog.hh"

#define Uses_TButton
#define Uses_TRect

#include <tvision/tv.h>

class MyTButton : public TButton {
	const HuiCallback	mCallback;

public:
	/**
	 * Constructor.
	 */
	MyTButton(
		const HuiRegion &region,
                const string &aTitle,
                ushort aFlags,
		const HuiCallback &aCallback)
	    : TButton(
		TRect(region.a.x, region.a.y, region.b.x, region.b.y),
		aTitle.c_str(),
		cmValid,
		aFlags),
	      mCallback(aCallback)
	{
	}

	void
	press()
	{
		mCallback.invoke();
		TButton::press();
	}
};

/**
 * Construct a TVision button object.
 */
HuiTvButton::HuiTvButton(
	HuiWidget *aParent,
	const HuiRegion &region,
	const string &aText,
	ushort flags,
	const HuiCallback &aCallback)
    : HuiTvEmbedded(aParent)
{
	mView = new MyTButton(
		region,
		aText,
		flags,
		aCallback);
}

/**
 * Destructor.
 */
HuiTvButton::~HuiTvButton()
{
	destroy(mView);
}

/**
 */
void
HuiTvButton::text(const string &aText)
{
	TButton *widget = dynamic_cast<TButton *>(mView);
	delete widget->title;
	widget->title = newStr(aText.c_str());
}

/**
 */
string
HuiTvButton::text() const
{
	TButton *widget = dynamic_cast<TButton *>(mView);
	return widget->title;
}

