/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiTv/HuiTvDialog.cc,v 1.37 2002/08/20 10:41:42 okumoto Exp $
 */

#include "HuiTv/HuiTv.hh"
#include "HuiTv/HuiTvDialog.hh"

#define Uses_TDeskTop
#define Uses_TDialog
#define Uses_TEvent

#include <tvision/tv.h>

class HuiTDialog : public TDialog {
	HuiTvDialog	&mDialog;
	string		mResult;
	TView		*mCurrent;

public:
	HuiTDialog(
		HuiTvDialog &aDialog,
		const HuiRegion &region,
		const string &aTitle)
	    : TDialog(
			TRect(region.a.x, region.a.y, region.b.x, region.b.y),
			aTitle.c_str()),
	      TWindowInit(&TDialog::initFrame),
	      mDialog(aDialog),
	      mCurrent(0)
	{
	}

	void
	handleEvent(TEvent &event)
	{
		if (event.what == evCommand &&
		    event.message.command == cmCancel) {
			clearEvent(event);
			mResult = "CLOSE";
			endModal(event.message.command);
		}

		bool resize_event = (event.what == evCommand) &&
				    ((event.message.command == cmResize) ||
				     (event.message.command == cmZoom));

		TDialog::handleEvent(event);

		if (mCurrent != current) {
			mDialog.aboutToChangeCurrent(mCurrent, current);
			mCurrent = current;
		}
		if (resize_event)
			mDialog.resized();
	}
};

HuiTvDialog::HuiTvDialog(
	const string &aTitle,
	const string &aAlignment,
	const HuiRegion &region)
    : HuiTvToplevel()
{
	mView = new HuiTDialog(*this, region, aTitle);
}

HuiTvDialog::~HuiTvDialog()
{
	TDialog *dialog = dynamic_cast<TDialog *>(mView);
	aboutToChangeCurrent(dialog->current, NULL);
	delete dialog;
}

string
HuiTvDialog::result() const
{
	return mResult;
}

string
HuiTvDialog::exec()
{
	assert(0);
	return mResult;
}

void
HuiTvDialog::endModal(const string &aResult)
{
	TDialog *dialog = dynamic_cast<TDialog *>(mView);

	dialog->endModal(cmDefault);
	mResult = aResult;
}

