/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiTv/HuiTvEmbedded.cc,v 1.40 2002/08/20 10:41:42 okumoto Exp $
 */

#include "HuiTv/HuiTvEmbedded.hh"

#define Uses_TGroup
#define Uses_TLabel
#define Uses_TParamText
#define Uses_TWindow

#include <tvision/tv.h>

/**
 * Constructor.
 */
HuiTvEmbedded::HuiTvEmbedded(
	HuiWidget *aParent)
    : HuiTvWidget(aParent),
      mTitle(NULL)
{
}

/**
 * Destructor.
 */
HuiTvEmbedded::~HuiTvEmbedded()
{
}

/**
 * Set the title for the HuiTvEmbedded.
 */
void
HuiTvEmbedded::title(const string &aTitle)
{
	string tmp = HuiTvEmbedded::translateAccel(aTitle);
	if (mTitle) {
		mTitle->setText(const_cast<char *>(tmp.c_str()));
	} else {
		//assert(0);
	}
}

string
HuiTvEmbedded::title() const
{
	if (mTitle) {
		char tmp[mTitle->getTextLen() + 1];
		mTitle->getText(tmp);
		return tmp;
	} else {
		return string();
	}
}

/**
 * Move the HuiTvEmbedded to the new position.
 */
void
HuiTvEmbedded::move(const Position &to)
{
	if (mTitle) {
		assert(0);
	} else {
		return HuiTvWidget::move(to);
	}
}

void
HuiTvEmbedded::resize(const Size &to, bool is_arranging = false)
{
	if (mTitle) {
		return HuiTvWidget::resize(to - Size(0, 1), is_arranging);
	} else {
		return HuiTvWidget::resize(to, is_arranging);
	}
}

Position
HuiTvEmbedded::position() const
{
	if (mTitle) {
		assert(0);
		return Position();
	} else {
		return HuiTvWidget::position();
	}
}

Size
HuiTvEmbedded::size() const
{
	if (mTitle) {
		return HuiTvWidget::size() + Size(0, 1);
	} else {
		return HuiTvWidget::size();
	}
}

