/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiTv/HuiTvGroup.cc,v 1.44 2002/08/22 11:14:57 okumoto Exp $
 */

#include "Hui/Hui.hh"
#include "HuiTv/HuiTvButton.hh"
#include "HuiTv/HuiTvCheckBoxes.hh"
#include "HuiTv/HuiTvContainer.hh"
#include "HuiTv/HuiTvGroup.hh"
#include "HuiTv/HuiTvInputLine.hh"
#include "HuiTv/HuiTvLabel.hh"
#include "HuiTv/HuiTvListBox.hh"
#include "HuiTv/HuiTvProgressIndicator.hh"
#include "HuiTv/HuiTvRadioMenu.hh"
#include "HuiTv/HuiTvTextEdit.hh"

#define Uses_TButton
#define Uses_TGroup
#include <tvision/tv.h>

/* insert GroupWidget */
void
HuiTvGroup::insert(HuiWidget *aWidget)
{
	HuiTvWidget	*w = dynamic_cast<HuiTvWidget *>(aWidget);
    	TGroup		*g = dynamic_cast<TGroup *>(mView);
	TView		*old_current = g->current;

	/* Also TVision add it */
	g->insert(w->widget());

	/* Add widget to list */
	mWidgets.push_back(w);

	/* Give focus to new widget */
	aboutToChangeCurrent(old_current, g->current);
}

HuiButton *
HuiTvGroup::button(
	const string &aText,
	const string &aButtonType,
	const HuiCallback &cb,
	const HuiRegion &region)
{
	string title = HuiTvEmbedded::translateAccel(aText);
	ushort flags = (aButtonType == "default") ? bfDefault : bfNormal;

	HuiTvButton *button =
		new HuiTvButton(
			this,
			region,
			title,
			flags,
			cb);
	insert(button);

	arrange();
	button->show();

	mWidgets.push_back(button);
	return button;
}

HuiCheckBoxes *
HuiTvGroup::checkBoxes(
	const vector<string> &aTexts,
	const vector<bool> &aInitialValues,
	const string &aTitle,
	const Position &aPosition,
	const vector<string> &aSVector)
{
	HuiTvCheckBoxes *checkbox =
		new HuiTvCheckBoxes(
			this,
			aTexts,
			aInitialValues,
			aTitle,
			aSVector);
	insert(checkbox);

	checkbox->move(aPosition);
	arrange();
	checkbox->show();

	mWidgets.push_back(checkbox);
	return checkbox;
}

HuiContainer *
HuiTvGroup::container(
	const Position &aPosition,
	const string &aAlignment)
{
	HuiTvContainer *container =
		new HuiTvContainer(
			this,
			aPosition,
			Size(0, 0));
	insert(container);

	container->move(aPosition);
	/* keep this order: */
	container->arrange(aAlignment, false);
	arrange();
	container->show();

	mWidgets.push_back(container);
	return container;
}

HuiInputLine *
HuiTvGroup::inputLine(
	const string &aInitialText,
	int aMaxLength,
	const string &aValidator,
	const string &aTitle,
	const Position &aPosition)
{
	HuiTvInputLine *inputline =
		new HuiTvInputLine(
			this,
			aInitialText,
			aMaxLength,
			aValidator,
			aTitle);
	insert(inputline);

	inputline->move(aPosition);
	arrange();
	inputline->show();

	mWidgets.push_back(inputline);
	return inputline;
}

HuiLabel *
HuiTvGroup::label(
	const string &aText,
	const HuiRegion &region)
{
	HuiTvLabel *label =
		new HuiTvLabel(
			this,
			aText,
			region);
	insert(label);
	label->show();

	mWidgets.push_back(label);
	return label;
}

HuiListBox *
HuiTvGroup::listBox(
	const HuiStringList &items,
	const int aInitiallySelected,
	const string &aTitle,
	const HuiRegion &region)
{
	HuiTvListBox *listbox =
		new HuiTvListBox(
			this,
			aTitle,
			items,
			aInitiallySelected,
			region);

	insert(listbox);
	listbox->show();

	mWidgets.push_back(listbox);
	return listbox;
}

HuiProgressIndicator *
HuiTvGroup::progressIndicator(
	float aMin,
	float aMax,
	float aInitial,
	const string &aTitle,
	const Position &aPosition)
{
	HuiTvProgressIndicator *progress =
		new HuiTvProgressIndicator(
			this,
			aMin,
			aMax,
			aInitial,
			aTitle);
	insert(progress);

	progress->move(aPosition);
	arrange();
	progress->show();

	mWidgets.push_back(progress);
	return progress;
}

HuiRadioMenu *
HuiTvGroup::radioMenu(
	const vector<string> &aTexts,
	const int aInitialValue,
	const string &aTitle,
	const Position &aPosition,
	const vector<string> &aSVector)
{
	HuiTvRadioMenu *radiomenu =
		new HuiTvRadioMenu(
			this,
			aTexts,
			aInitialValue,
			aTitle,
			aSVector);
	insert(radiomenu);

	radiomenu->move(aPosition);
	arrange();
	radiomenu->show();

	mWidgets.push_back(radiomenu);
	return radiomenu;
}

HuiTextEdit *
HuiTvGroup::textEdit(
	const string &aInitialText,
	bool aAllowEdit,
	const string &aTitle,
	const HuiRegion &region)
{
	HuiTvTextEdit *textedit =
		new HuiTvTextEdit(
			this,
			aInitialText,
			aAllowEdit,
			aTitle,
			region);
	insert(textedit);

	arrange();
	textedit->show();

	mWidgets.push_back(textedit);
	return textedit;
}

void
HuiTvGroup::arrange()
{
	if (mArrangement == ARRANGE_FREE)
		return;

	TPoint mysize;
	mysize.x = size().width;
	mysize.y = size().height;

	/*
	 * First, calculate how much of the given space has to be
	 * shared by widgets w/o fixed size
	 */
	int sharedsize = (mArrangement == ARRANGE_VERTICAL) ?
					mysize.y : mysize.x;
	int numwidgets = 0;
	WidgetList::iterator sw;
	for (sw = mWidgets.begin(); sw != mWidgets.end(); ++sw) {
		int	widgetfixedsize = (*sw)->mFixedSize;
		if (widgetfixedsize == -1) {
			numwidgets++;
		} else {
			sharedsize -= widgetfixedsize;
		}
	}

	if (mArrangement == ARRANGE_HORIZONTAL) {
		int ipos = 0;

		for (sw = mWidgets.begin(); sw != mWidgets.end(); ++sw) {
			HuiTvWidget	*w = (*sw);

			/* Check, if the current widget wants a fixed size */
			int	widgetfixedsize = w->mFixedSize;
			TPoint	newpos;
			newpos.x = ipos;
			newpos.y = 0;

			TPoint newsize;
			if (widgetfixedsize == -1) {
				/* Calculate new size */
				int newwidth = sharedsize / numwidgets;
				newsize.x = newwidth;
				newsize.y = mysize.y;
				ipos += newwidth;
				numwidgets--;
				sharedsize -= newwidth;
			} else {
				/* If so, just move and resize it */
				newsize.x = widgetfixedsize;
				newsize.y = mysize.y;
				ipos += widgetfixedsize;
			}

			HuiTvContainer *cp =
				dynamic_cast<HuiTvContainer *>(w);
			HuiTvWidget *gp =
				dynamic_cast<HuiTvWidget *>(w);

			if (cp != NULL) {
				cp->move(Position(newpos.x, newpos.y));
				cp->resize(Size(newsize.x, newsize.y), true);
			}
			if (gp != NULL) {
				gp->move(Position(newpos.x, newpos.y));
				gp->resize(Size(newsize.x, newsize.y), true);
			}
		}
	}
	if (mArrangement == ARRANGE_VERTICAL) {
		int ipos = 0;

		for (sw = mWidgets.begin(); sw != mWidgets.end(); ++sw) {
			HuiTvWidget	*w = (*sw);

			/* Check, if the current widget wants a fixed size */
			int	widgetfixedsize = w->mFixedSize;
			TPoint	newpos;
			newpos.x = 0;
			newpos.y = ipos;
			TPoint newsize;

			if (widgetfixedsize == -1) {
				/* Calculate new size */
				int newheight = sharedsize / numwidgets;
				newsize.x = mysize.x;
				newsize.y = newheight;
				ipos += newheight;
				numwidgets--;
				sharedsize -= newheight;
			} else {
				/* If so, just move and resize it */
				newsize.x = mysize.x;
				newsize.y = widgetfixedsize;
				ipos += widgetfixedsize;
			}

			HuiTvContainer *cp =
				dynamic_cast<HuiTvContainer*>(w);
			HuiTvWidget *gp =
				dynamic_cast<HuiTvWidget*>(w);

			if (cp != NULL) {
				cp->move(Position(newpos.x, newpos.y));
				cp->resize(Size(newsize.x, newsize.y), true);
			}
			if (gp != NULL) {
				gp->move(Position(newpos.x, newpos.y));
				gp->resize(Size(newsize.x, newsize.y), true);
			}
		}
	}
}

void
HuiTvGroup::arrange(
	const string &aAlignment,
	bool arrange_new = true)
{
	if (aAlignment == "horizontal")
		mArrangement = ARRANGE_HORIZONTAL;
	else if (aAlignment == "vertical")
		mArrangement = ARRANGE_VERTICAL;
	else if (aAlignment == "free")
		mArrangement = ARRANGE_FREE;
	else if (aAlignment != "")
		throw Hui::Error(string("Invalid alignment  `") + aAlignment + "' (`horizontal', `vertical' or `free' expected)");

	if (arrange_new)
		arrange();
}

/* get current widget (focused) */
HuiWidget *
HuiTvGroup::current() const
{
	/*
	 * Call TVision function to get the current, then search for this one
	 * in the group of sub widgets
	 */
	TGroup	*g = dynamic_cast<TGroup *>(widget());

	for (WidgetList::const_iterator w = mWidgets.begin(); w != mWidgets.end(); ++w) {
		if ((*w)->widget() == g->current) {
			return (*w);
		}
	}
	return NULL;
}

/* change focus between two widgets */
void
HuiTvGroup::aboutToChangeCurrent(TView* old, TView* now)
{
	if (old != now) {
		HuiWidget* cur = NULL;
		for (WidgetList::const_iterator w = mWidgets.begin(); w != mWidgets.end(); ++w) {
			if ((*w)->widget() == old) {
				cur = (*w);
			}
		}

		HuiWidget* ncur = NULL;
		for (WidgetList::const_iterator w = mWidgets.begin(); w != mWidgets.end(); ++w) {
			if ((*w)->widget() == now) {
				cur = (*w);
			}
		}

#if 0
		if (ncur) {
			if (cur)
				cur->leave();
			ncur->enter();
		}
#else
		// do nothing
		//assert(0);
#endif
	}
}

/* set current Widget (focus) */
void
HuiTvGroup::current(HuiEmbedded *aWidget)
{
	HuiTvEmbedded *w =
		dynamic_cast<HuiTvEmbedded*>((HuiEmbedded *)aWidget);

    	TGroup	*g = dynamic_cast<TGroup *>(widget());
	TView	*old_current = g->current;

	g->setCurrent(w->widget(), TView::normalSelect);

	aboutToChangeCurrent(old_current, g->current);
}

/* Remove widget */
void
HuiTvGroup::erase()
{
	WidgetList::iterator sw;

	for (sw = mWidgets.begin(); sw != mWidgets.end(); ++sw) {
		HuiWidget *w = (*sw);
#if 1
		w->hide();
#else
		ContainerWidget *cp =
			dynamic_cast<HuiTvContainer*>(w);
		HuiTvWidget *gp =
			dynamic_cast<HuiTvWidget*>(w);
		HuiTvEmbedded *tp =
			dynamic_cast<HuiTvEmbedded*>(w);

		if (cp != NULL) {
			cp->erase();
		} else if (tp != NULL) {
			widget()->remove((TLabel *) tp->ttitle());
			widget()->remove(tp->widget());
		} else if (gp != NULL) {
			widget()->remove(gp->widget());
		}
		/* XXX this should also remove eventual scrollbars */
#endif
	}

	mWidgets.clear();
}

