/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiTv/HuiTvMenuSubmenu.cc,v 1.37 2002/08/20 10:41:42 okumoto Exp $
 */

#include "HuiTv/HuiTvEmbedded.hh"
#include "HuiTv/HuiTvMenuBar.hh"
#include "HuiTv/HuiTvMenuButton.hh"
#include "HuiTv/HuiTvMenuSeparator.hh"
#include "HuiTv/HuiTvMenuSubmenu.hh"

#define Uses_TMenu
#define Uses_TMenuItem
#define Uses_TSubMenu

#include <tvision/tv.h>

void
HuiTvMenuSubmenu::append(HuiTvMenuItem *item)
{
	if (mMenuItem->subMenu == NULL) {
		mMenuItem->subMenu = new TMenu(*item->mwidget());
	} else {
		/* Append new submenu to end of list */
		for (TMenuItem *i = mMenuItem->subMenu->items; i != NULL; i = i->next) {
			if (i->next == NULL) {
				i->append(item->mwidget());
				break;
			}
		}
	}

	wl.push_back(item);
}

/**
 * Construct an empty TVision SubMenu.  This is called by the
 * HuiTvMenuBar constructor.  It does not create a TSubMenu since
 * the MenuBar already draws itself.
 */
HuiTvMenuSubmenu::HuiTvMenuSubmenu(
	HuiTvMenuBar &aBar)
    : HuiTvMenuItem(NULL),
      bar(aBar)
{
      mText = "";
}

/**
 * Construct an empty TVision SubMenu.
 */
HuiTvMenuSubmenu::HuiTvMenuSubmenu(
	HuiWidget *aParent,
	HuiTvMenuBar &aBar,
	const string &aText,
	const string &aKey)
    : HuiTvMenuItem(aParent),
      bar(aBar)
{
	string title = HuiTvEmbedded::translateAccel(aText);
	ushort key = key2keyCode(aKey);

	mMenuItem = new TSubMenu(
			title.c_str(),
			key);
}

/**
 */
HuiMenuItem *
HuiTvMenuSubmenu::button(
	const string &aText,
	const string &aKey,
	const HuiCallback &callback,
	HuiMenuItem *aBefore)
{
	string title = HuiTvEmbedded::translateAccel(aText);
	ushort key = key2keyCode(aKey);

	HuiTvMenuItem *item =
		new HuiTvMenuButton(
			this,
			title,
			bar.addCmd(callback),
			key);

	append(item);
	return item;
}

/**
 */
HuiMenuItem *
HuiTvMenuSubmenu::separator(
	HuiMenuItem *aBefore)
{
	HuiTvMenuItem *item =
		new HuiTvMenuSeparator(
			this);

	append(item);
	return item;
}

/**
 */
HuiMenuSubmenu *
HuiTvMenuSubmenu::submenu(
	const string &aText,
	const string &aKey,
	HuiMenuItem *aBefore)
{
	HuiTvMenuSubmenu *item =
		new HuiTvMenuSubmenu(
			this,
			bar,
			aText,
			aKey);

	append(item);
	return item;
}

