/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiTv/HuiTvProgressIndicator.cc,v 1.38 2002/08/20 10:41:42 okumoto Exp $
 */

#include "HuiTv/HuiTvProgressIndicator.hh"

#define Uses_TView
#define Uses_TRect

#include <tvision/tv.h>

class HuiTProgressIndicator : public TView {
	float mMin;
	float mMax;
	float mCurrent;
	char mTodoSymbol;
	char mDoneSymbol;
public:
	HuiTProgressIndicator(
		const TRect &bounds,
		float aMin,
		float aMax,
		char aTodoSymbol,
		char aDoneSymbol)
	    : TView(bounds),
	      mMin(aMin),
	      mMax(aMax),
	      mCurrent(aMin),
	      mTodoSymbol(aTodoSymbol), mDoneSymbol(aDoneSymbol)
	{
	}

	virtual void
	draw()
	{
		const ushort todoColor = getColor(1), doneColor = getColor(2);

		const size_t doneSize = size_t(mCurrent / (mMax - mMin) * float(size.x));
		const size_t todoSize = size.x - doneSize;

		TDrawBuffer b;
		for (short i = 0; i < size.y; ++i) {
			b.moveChar(0, mDoneSymbol, doneColor, doneSize);
			b.moveChar(doneSize, mTodoSymbol, todoColor, todoSize);
			writeLine(0, i, size.x, 1, b);
		}
	}

#define cpProgressIndicator "\x0D\x13"
	virtual TPalette &
	getPalette() const
	{
		static TPalette palette(cpProgressIndicator, sizeof(cpProgressIndicator) - 1);
		return(palette);
	}

	virtual void
	set(float aCurrent)
	{
		if (aCurrent < mMin)
			mCurrent = mMin;
		else if (aCurrent > mMax)
			mCurrent = mMax;
		else
			mCurrent = aCurrent;
		drawView();
	}

	virtual float
	set() const
	{
		return(mCurrent);
	}
};

HuiTvProgressIndicator::HuiTvProgressIndicator(
	HuiWidget *aParent,
	float aMin,
	float aMax,
	float aInitial,
	const string &aTitle)
    : HuiTvEmbedded(aParent)
{
	mView = new HuiTProgressIndicator(TRect(1, 1, 11, 3),
						   aMin, aMax, '-', '>');

	set(aInitial);
	resize(Size(4, 1 + (ttitle() ? 1 : 0)));
}

HuiTvProgressIndicator::~HuiTvProgressIndicator()
{
	delete mView;
}

void
HuiTvProgressIndicator::set(float aValue)
{
	HuiTProgressIndicator *widget = dynamic_cast<HuiTProgressIndicator *>(mView);
	widget->set(aValue);
}

float
HuiTvProgressIndicator::set() const
{
	HuiTProgressIndicator *widget = dynamic_cast<HuiTProgressIndicator *>(mView);
	return widget->set();
}

