/*-
 * Copyright (c) 1998-2002 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/HuiTv/HuiTvWidget.cc,v 1.37 2002/08/20 10:41:42 okumoto Exp $
 */

#include "HuiTv/HuiTvWidget.hh"
#include "HuiTv/HuiTvGroup.hh"

#define Uses_TWindow

#include <tvision/tv.h>

/**
 * Constructor.
 *
 * @param parent	Pointer to widget containing this object,
 *			else NULL.
 */
HuiTvWidget::HuiTvWidget(HuiWidget *parent)
    : mParent(parent),
      mView(NULL),
      mFixedSize(-1)
{
}

/**
 * Move the widget to the new position.
 */
void
HuiTvWidget::move(const Position &pos)
{
	TWindow *widget = dynamic_cast<TWindow *>(mView->owner);
	if ((widget == NULL) || (widget->frame == NULL)) {
		mView->moveTo(pos.x, pos.y);
	} else {
		mView->moveTo(pos.x + 1, pos.y + 1);
	}
}

/**
 * Resize the widget.
 */
void
HuiTvWidget::resize(const Size &aSize, bool is_arranging = false)
{
	Dimension	w = aSize.width;
	Dimension	h = aSize.height;

	if (w == -1 && h == -1)
		return; /* don't do anything, widget size is fixed */

	if (w >= 0 && h >= 0) {
		if (widget())
			widget()->growTo(w, h);
		return;	/* change the size of the TVision object */
	}

	if (is_arranging) {
		/* XXX shouldn't this be at the top of the function? */
		return; /* don't do stuff yet */
	}

	if (w >= 0 || h >= 0) {
		mFixedSize = (w < 0) ? h : w;

		/* arrange parent again since we changed the fixed size */
		HuiTvGroup *g = dynamic_cast<HuiTvGroup *>(mParent);
		g->arrange();
	}
}

Position
HuiTvWidget::position() const
{
	return Position(mView->origin.x, mView->origin.y);
}

Size
HuiTvWidget::size() const
{
	return Size(mView->size.x, mView->size.y);
}

void
HuiTvWidget::show()
{
	mView->show();
}

void
HuiTvWidget::hide()
{
	mView->hide();
}

bool
HuiTvWidget::hidden() const
{
	return mView->getState(sfVisible);
}

void
HuiTvWidget::disable()
{
	mView->setState(sfDisabled, True);
}

void
HuiTvWidget::enable()
{
	mView->setState(sfDisabled, False);
}

bool
HuiTvWidget::disabled() const
{
	return (mView->getState(sfDisabled) == True);
}

