/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/Hui.cc,v 1.13 2002/05/08 09:02:36 okumoto Exp $
 */

// -*- C++ -*-   Tue Dec  1 1998
// Eugene Skepner <eu@qub.com>
// $Id: Hui.cc,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#include "hui/Hui.hh"

#ifdef USE_Qt
#include "hui/HuiQtNew.hh"
#endif

#ifdef TVision
#include "hui/HuiTVisionNew.hh"
#endif

#include <cstdlib>

Hui::Hui_P
Hui::open(const string &aType)
{
    if (aType == "auto")
	return open(tAuto);
    else if (aType == "text")
	return open(tText);
    else if (aType == "graphics")
	return open(tGraphics);
    else
	throw Error("Wrong Hui type `" + aType +
		    "' (possible values: auto, text, graphics)");
}

Hui::Hui_P
Hui::open(Type type)
{
#ifdef USE_Qt
    if (type == tAuto) {
	if (getenv("DISPLAY"))
	    type = tGraphics;
	else
	    type = tText;
    }
#else
    type = tText;
#endif

    Hui_P	hui;
    switch (type) {
    case tAuto:
	break;

    case tText:
#ifdef TVision
	hui = newHuiTVision();
#else
	throw Error("Hui Text type is not supported by this build of the library");
#endif
	break;

    case tGraphics:
#ifdef USE_Qt
	{
	    char* argv[] = { "Error:", "-geometry", "+0+0" /*"500x300"*/ };
	    hui = newHuiQt(sizeof(argv) / sizeof(argv[0]), argv);
	}
#else
	throw Error("Hui Graphics type is not supported by this build of the library");
#endif
	break;
    }

#if 0
    if (hui && !strcmp(progressIndicatorStack->objecttypename(), "HuiProgressIndicatorStack"))
	dynamic_cast<HuiProgressIndicatorStack *>(progressIndicatorStack)->hui(hui);
#endif
    return(hui);
}

