/*-
 * Copyright (c) 2002 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $FreeBSD: libh/lib/part/fstab_file.cc,v 1.5 2002/04/22 18:06:25 antoine Exp $
 */

extern "C" {
#include <fstab.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/mount.h>
#include <ufs/ufs/ufsmount.h>
}

#include <cerrno>
#include <sstream>

#include "part/fstab_file.hh"

//======================================================================

cfstab::cfstab(fstab &f) : fs_spec(f.fs_spec), 
			   fs_file(f.fs_file), 
			   fs_vfstype(f.fs_vfstype), 
			   fs_mntops(f.fs_mntops),
			   fs_type(f.fs_type),
			   fs_freq(fs_freq),
			   fs_passno(fs_passno)
 {}

fstab_file::fstab_file() : cfstabs() {

	fstab *f;
	cfstab f2;

 	while ( (f = ::getfsent()) != NULL) {
		f2 = cfstab(*f);
		//   		cf = cfstab(*f);
   		cfstabs.push_back(f2);
 	}
}

const string cfstab::str() {
	ostringstream s;
	s << fs_spec << " " << fs_file << " " << fs_vfstype << " "
	  << fs_mntops << " " << fs_freq << " " << fs_passno;
	return s.str();
}

void cfstab::mount() {

	struct ufs_args ufsargs;

	::memset(&ufsargs,0,sizeof ufsargs);

	ufsargs.fspec = const_cast<char*>(fs_spec.c_str());
	if (::mount(fs_vfstype.c_str(), fs_file.c_str(), 0,
		  (caddr_t)&ufsargs) == -1) {
		throw runtime_error("Error mounting " + fs_vfstype + " on " +
			    fs_file + " : " + strerror(errno));
	}
}

// ostream& operator<< (ostream& st, const cfstab &cfs) {

// 	return st << cfs.fs_spec << " " << fs_file << " " << fs_vfstype << " "
// 		  << cfs.fs_mntops << " " << fs_freq << " " << fs_passno;

// }
