/*-
 * Copyright (c) 1998-2002 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/tcl/tcl_interface_gen.cc,v 1.53 2002/04/24 11:05:03 okumoto Exp $
 */

// Eugene Skepner <eu@qub.com>
// $Id: tcl_interface_gen.cc,v 1.45 1999/01/06 15:17:48 eugene Exp $

#include "HSystem.hh"
#include "String.hh"

#include <fstream>
#include <stdexcept>
#include <cerrno>
#include <ctime>
#include <cstdio>

/**
 * Main entry point for tcl bindings generator.  Create the interface code
 * between tcl and C++
 *
 * @returns	0 for success,
 *		1 for incorrect number of args,
 *		2 for some an exception.
 */
int
main(int argc, char** argv)
{
	const string	progname = argv[0];

	if (argc != 3) {
		cerr << "Usage: " << progname << " <system-name> <file-to-generate>" << endl;
		return 1;
	}

	const string	name = argv[1];
	const string	ofile = argv[2];

	try {
		Systems systems;
		build_systems(systems);

		Systems::const_iterator system = systems.find(HSystem(name));
		if (system == systems.end())
			throw runtime_error("System " + name + " is unknown");

		ofstream out(ofile.c_str());
		if (!out)
			throw runtime_error("Unable to create " + ofile +
					    ": " + ::strerror(errno));
		system->gen_file(out);
	}
	catch (exception& err) {
		cerr << "Error: " << err.what() << endl;
		return 2;
	}

	return 0;
}

