# $FreeBSD: libh/release/floppies/Makefile.inc,v 1.11 2002/04/11 18:37:29 antoine Exp $

# XXX: qt definitly doesn't fit on floppies
NOQT		=	yes

# for UITYPE
.include "../../Makefile.uitype"

# File names
MFS_NAME	=	mfsroot
MFSFL_NAME	=	mfsroot.flp
KERN_NAME	=	kern.flp
BOOT_NAME	=	boot.flp

MFS_VNODE	=	0
MFSFL_VNODE	=	${MFS_VNODE}
KERN_VNODE	=	${MFS_VNODE}
BOOT_VNODE	=	${MFS_VNODE}

MFS_MP		=	mmnt
MFSFL_MP	=	fmnt
KERN_MP		=	kmnt
BOOT_MP		=	bmnt

ECHO_MSG	=	@echo "===>"

# Get __FreeBSD_version
# stolen from bsd.port.mk
.if !defined(OSVERSION)
.if exists(/sbin/sysctl)
OSVERSION!=     /sbin/sysctl -n kern.osreldate
.else
OSVERSION!=     /usr/sbin/sysctl -n kern.osreldate
.endif
.endif

.if ${OSVERSION} < 500000
MDCONFIG	?=	vnconfig -s labels -c
MDUNCONFIG	?=	vnconfig -u
MDEV		?=	/dev/vn
MUNIT		?=	${MDEV}
.else
MDCONFIG	?=	mdconfig -a -t vnode
MDUNCONFIG	?=	mdconfig -d
MDEV		?=	/dev/md
MUNIT		?=	"-u "
.endif

# Information about mfs parameters
# Size of mfs in kilobytes
MFS_SIZE	=	4500

# System source
SRC		=	/usr/src
OBJ		?=	/usr/obj

# Name of init program
INITDIR		=	${.CURDIR}/../init
INIT		=	init

# Tclh
TCLHDIR		=	${.CURDIR}/../../compile/${UITYPE}/
TCLH		=	tclh.static

# Variables for building kernel
MACHINE		?=	i386
MACHINE_ARCH	?=	i386
CONFDIR		?=	$(SRC)/sys/${MACHINE}/conf
GEN_CONFFILE	=	GENERIC
CONFFILE	=	LIBH-BOOT
KERNFILE	=	$(COMPILE)/kernel

FDSIZE		=	BIG

# we have 2 ways of compiling kernels
.if defined (CONFIG_WAY)
# the old "config" way
COMPILE		=	$(SRC)/sys/compile/$(CONFFILE)
.else
# and the new "buildkernel" way
COMPILE		=	$(OBJ)/$(SRC)/sys/$(CONFFILE)
.endif

# Number of bytes per inode in mfs
MFS_INODES	=	4096

# Number of bytes per inode in kern image
FLOPPY_INODES	=	32768

# Suffix to tack on somewhere
suffix		=	-C

# Common targets

vnconfig-mfs:
	${MDCONFIG} ${MUNIT}${MFS_VNODE} ${MFS_NAME}
vnconfig-mfsfl:
	${MDCONFIG} ${MUNIT}${MFSFL_VNODE} ${MFSFL_NAME}
vnconfig-kern:
	${MDCONFIG} ${MUNIT}${KERN_VNODE} ${KERN_NAME}

vnconfig-boot:
	${MDCONFIG} ${MUNIT}${MFS_VNODE} ${MFS_NAME}

vnunconfig-mfs:
	${MDUNCONFIG} ${MUNIT}${MFS_VNODE}
vnunconfig-mfsfl:
	${MDUNCONFIG} ${MUNIT}${MFSFL_VNODE}
vnunconfig-kern:
	${MDUNCONFIG} ${MUNIT}${KERN_VNODE}
vnunconfig-boot:
	${MDUNCONFIG} ${MUNIT}${BOOT_VNODE}

mount-mfs: ${MFS_MP}
	mount ${MDEV}${MFS_VNODE} ${MFS_MP}
mount-mfsfl: ${MFSFL_MP}
	mount ${MDEV}${MFSFL_VNODE} ${MFSFL_MP}
mount-kern: ${KERN_MP}
	mount ${MDEV}${KERN_VNODE} ${KERN_MP}
mount-boot: ${BOOT_MP}
	mount ${MDEV}${BOOT_VNODE} ${BOOT_MP}

umount-mfs:
	umount ${MDEV}${MFS_VNODE}
umount-mfsfl:
	umount ${MDEV}${MFSFL_VNODE}
umount-kern:
	umount ${MDEV}${KERN_VNODE}
umount-boot:
	umount ${MDEV}${BOOT_VNODE}

${MFS_MP} ${MFSFL_MP} ${KERN_MP} ${BOOT_MP}:
	mkdir -p $@
