#
# Dialog.tcl --
#
#	This file implements the Tcl code for creating and
#	managing the UI interface for libh.
#
# Copyright (c) 2002 The FreeBSD Project
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation
# are those of the authors and should not be interpreted as representing
# official policies, either expressed or implied, of the FreeBSD Project.
#
# $FreeBSD: libh/release/scripts/Dialog.tcl,v 1.13 2002/08/26 11:04:55 okumoto Exp $
#

package provide Dialog 0.1
namespace eval Dialog {
    namespace export buildUI

    #
    # buildUI --
    #
    #	Create the main window.
    #
    # Arguments:
    #	hui	The libh UI object.
    #
    # Results:
    #	None.
    #

    proc buildUI {hui} {
	#make_main_win $hui
	make_sec_win $hui

	$hui run
    }

    proc next_callback {arg} {
	puts Hello
    }

    proc menu_callback {arg} {
	puts Bye
	puts $arg
    }

    proc make_menu_bar {window} {
	# Create menu bar
	set menuBar [$window menuBar]
	HuiMenuSubmenu -this $menuBar

	# Populate menu bar
	set m10 [$menuBar submenu "&A Menu" "" NULL]
	set m20 [$menuBar submenu "&B Menu" "" NULL]
	set m30 [$menuBar submenu "&C Menu" "" NULL]
	HuiMenuSubmenu -this $m10
	HuiMenuSubmenu -this $m20
	HuiMenuSubmenu -this $m30

	set m11 [$m10 submenu "&D Menu" "" NULL]
	set m12 [$m10 submenu "&E Menu" "" NULL]
	set m13 [$m10 submenu "&F Menu" "" NULL]

	set m21 [$m20 submenu "&G Menu" "" NULL]
	set m22 [$m20 submenu "&H Menu" "" NULL]
	set m23 [$m20 submenu "&I Menu" "" NULL]

	set m31 [$m30 button "&J Button" "" {menu_callback "j"} NULL]
	set m32 [$m30 separator NULL]
	set m33 [$m30 button "&K Button" "" {menu_callback "k"} NULL]
	set m34 [$m30 button "&L Button" "" {menu_callback "l"} NULL]
    }

    # Create the main-window and its elements
    proc make_main_win {hui} {
	set msg "Hardware Information
CPU	Pentium III/Pentium III Xeon/Celeron (696.41-MHz 686-class CPU)
Memory	671023104 (655296K bytes)
Drives	acd0, da0, da1, da2, da3, ad2
NIC	fxp0, fxp1

Software Information
Kernel	FreeBSD 4.5-RELEASE-p2 #0: Fri Apr  5 01:30:35 PST 2002
	root@takai.home:/data/freebsd/src/sys/compile/TAKAI
Libh	Version 0.1 Alpha
Media	CDROM, Network, floppy"

	set main_window [$hui window "FreeBSD Setup" "" {80 25 0 0}]
	HuiWindow -this $main_window

	set welcometext [$main_window textEdit $msg 0 "" {78 12 0 1}]

	set next_button [$main_window button "&Next ->" "normal" {next_callback ""} {20 4 1 20}]

	make_menu_bar $main_window
    }

    proc b1_callback {arg} {
	puts "B1 was pressed"
    }

    # Create the main-window and its elements
    proc make_sec_win {hui} {
	set sec_window [$hui window "All GUI Elements" "" {80 23 0 0}]
	HuiWindow -this $sec_window

#	make_menu_bar $sec_window

	$sec_window label "A Row" {5 1 1 2}

	$sec_window button "&Button 1 20%" "normal" {b1_callback ""} {19 2 1 3}
	$sec_window button "B&utton 2 30%" "normal" {b2_callback ""} {21 2 20 3}
	$sec_window button "Bu&tton 3 50%" "normal" {b3_callback ""} {25 2 41 3}

	$sec_window label "The Lists" {9 1 1 5}
}
proc foo {} {

#
# Note: We fake the widgets until they work.
#

	#$sec_window textEdit "A List(fake)" 0 ""	{16 5 2 6}
	$sec_window listBox {} -1 "A List"	{16 5 2 6}

	$sec_window textEdit "A Checklist(fake)" 0 ""	{22 5 21 6}
	#$sec_window label "A Checklist"	{22 5 21 6}

	$sec_window textEdit "A Raidolist(fake)" 0 ""	{22 5 45 6}
	#$sec_window label "A Raidolist"	{22 5 45 6}

	$sec_window label "Gauges" {6 1 1 11}
	$sec_window textEdit "  13% ###......................." 0 ""	{33 3 1 12}
	$sec_window textEdit "  50% #############............." 0 ""	{33 3 34 12}
    }
}
