#
# Hui.tcl --
#
#	This file implements the Tcl code for creating and
#	managing the UI interface for libh.
#
# Copyright (c) 2002 The FreeBSD Project
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation
# are those of the authors and should not be interpreted as representing
# official policies, either expressed or implied, of the FreeBSD Project.
#
# $FreeBSD: libh/release/scripts/Hui.tcl,v 1.4 2002/08/26 11:04:55 okumoto Exp $
#

package provide Hui 0.1
namespace eval Hui {
    namespace export initHui cleanupHui

    #
    # initHui --
    #
    #	Initialize user interface.  Check if the Hui code is
    #	built into the interp.
    #
    # Arguments:
    #	None.
    #
    # Results:
    #	Hui object.
    #

    proc initHui {bin_type dis_type} {
	if {$bin_type != "static"} {
	    load $bin_type
	}

	if {[info command HuiQt] == "HuiQt"} {
	    return [HuiQt -args 0 NULL]
	} elseif {[info command HuiTv] == "HuiTv"} {
	    return [HuiTv -args 0 NULL]
	} else {
	    puts "Can't find a Hui module."
	    exit 1
	}
    }

    #
    # cleanupHui --
    #
    #	Cleanup user interface.
    #
    # Arguments:
    #	hui	The libh UI object.
    #
    # Results:
    #	None.
    #

    proc cleanupHui {hui} {
	$hui -delete
    }

}
