#
# disklabel editor
#

# a possible technique to do this:
# hold a tcl list of the "partitions" (wether dos/whatever or fbsd part)
# and keep it in syn with the listbox list -> add, select, remove, etc ops
#
# see also display_chunk_r

#
# bugs from sysinstall not to repeat:
# - do not "overmount" without confirm, epecially / -> needs getmntinfo api
# - must quit sysinstall to make 2 writes in disklabel -> DONE
#

source scripts/diskedit/dialogs.tcl
source scripts/diskedit/formats.tcl

# walk a chunk architecture, exectuting proc on each chunk
# XXX: this does not belong here, but more to lib/disk/
# the procedure is called with the chunk arg and a "level"
# arg indicating the current depth in the tree
#
# see print_chunk for a sample call
proc recurse_chunk {chunk procname {level 0}} {
#     if {[info commands $procname] != $procname} {
#         puts stderr "unknown procedure $procname!!!"
#     }
    
    if {![H::is_null $chunk]} {
        $procname $chunk $level
        recurse_chunk [$chunk next] $procname $level
        recurse_chunk [$chunk part] $procname [expr $level + 1]
    }
}

# Debug: print the chunk on stdout
proc print_chunk {chunk level} {

    puts [format "%d %s %10s %10s %10s %10s" \
          $level $chunk [$chunk name] [$chunk type_text] \
              [$chunk offset] [$chunk end]]

}

#
# list setup callbacks
# 

# add a chunk to the parts list and to the part to disk match list
# expect a global current_disk containing the parent of that part
#
# we have to keep a list of the disk parents to these parts
#
# this is necessary to keep a certain level of sanity for partition
# operations such as "delete" or "add"
proc add_chunk {chunk {level 0}} {

    global parts_list
    global current_disk
    global part_to_disk_list
    if {([$chunk type] >= 0 && [$chunk type] < 6) ||
        ([$chunk type] == 6 && $level < 2)} {
        lappend parts_list $chunk
        lappend part_to_disk_list $current_disk
    }
}

# remake the parts list and the part to disk match list without 
# re-reading the disks
#
# recurses with add_chunk, basically
proc update_parts_list {} {

    global disk_list
    global current_disk
    global parts_list
    global part_to_disk_list

    set parts_list [list]
    set part_to_disk_list [list]
    foreach disk $disk_list {
        set current_disk $disk
        recurse_chunk [$disk chunks] "add_chunk"
    }
}

#
# listbox setup callbacks
#

# maybe add a chunk to the global "parts" listbox
#
# see the non-global proc display_chunk_r
proc display_chunk {chunk {level 0}} {

    global parts
    display_chunk_r $parts $chunk $level
    
}

# maybe add a chunk to the given listbox
#
# level determines wether or not a "unused" part is displayed:
# if level < 2, it means that the unused part is not within a
# freebsd slice, so we should show it
proc display_chunk_r {listbox chunk level} {
    
    global sector_size

    switch [$chunk type] {
        # whole
        0 {$listbox add [$listbox numberOfItems] [whole_format $chunk $sector_size]}
        # unknown
        1 {$listbox add [$listbox numberOfItems] [part_format $chunk $sector_size]}
        # fat
        2 {$listbox add [$listbox numberOfItems] [dos_format $chunk $sector_size]}
        # fbsd
        3 {$listbox add [$listbox numberOfItems] [fbsd_format $chunk $sector_size]}
        # extended
        4 {$listbox add [$listbox numberOfItems] [dos_format $chunk $sector_size]}
        # part
        5 {$listbox add [$listbox numberOfItems] [part_format $chunk $sector_size]}
        # unused
        6 {
            if {$level < 2} {
                $listbox add [$listbox numberOfItems] [unused_format $chunk $sector_size]
            }
        }
        default {}
    }
}


# clear the disk list and re-read it
#
# this is basically an undo implementation
proc read_disks {} {

    global disk_list
    
    # clear the list
    # XXX: do we have to free each node????
    set disk_list [list]

    # refill everything
    foreach disk_name [lsort [Disk::Disk_Names]] {
        if {![string match "*cd*" $disk_name] &&
            ![string match "*md*" $disk_name]
        } {
            if { [catch {set disk [Disk $disk_name]} fid] } {
                # XXX: we should put this in some kind of status bar
		#puts stderr "Warning: $fid"
            } else {
                lappend disk_list $disk
            }
        }
    }
}

proc min {a b} {return [expr $a < $b ? $a : $b]}

# main dialog creation code, should be called only once
proc disklabel {hui parent} {
    global parts
    global disk_window
    global size_units

    set size_units "Mb"

    set disk_window [$hui dialog $parent "labeldialog"\
                    "Partition Editor" "1 0" "80 19" "vertical"]
    
    # parts section
    set parts [$disk_window listBox "partslistbox" {} -1 \
                   "Flags\tPartition\tStart\tEnd\tType\tSize"]
    
    # lower buttons
    set but_cont [$disk_window container "but_cont" "0 0" "horizontal"]

    set m_buts [$but_cont container "m_buts" "0 0" "vertical"]
    $m_buts button "auto_t_but" "Auto c&reate" {parts_callback AUTO}
    $m_buts button "boot_but" "Toggle &Bootable" {parts_callback BOOT}
    # change type? that's a weird idea
    # let's just assume the user will delete + add instead
    # in order to have a "change type" button, we'll have to have a 
    # "replace" button and/or function and use it to implement this
#     $m_buts button "type_but" "Change &Type" {parts_callback TYPE}
    $m_buts button "units_but" "Cycle Si&ze Units ($size_units)" {cycle_units_callback ""}
    $m_buts button "undo_but" "&Undo changes" {parts_callback UNDO}

    set r_buts [$but_cont container "r_buts" "0 0" "vertical"]
    $r_buts button "add_but" "&Add" {parts_callback ADD}
    # we postpone this implementation until after primary impl (add/del)
#     $r_buts button "res_but" "&Resize" {parts_callback RESIZE}
    $r_buts button "del_but" "&Delete" {parts_callback DELETE}
    $r_buts button "edit_but" "&Edit" {parts_callback EDIT}
    $r_buts button "write_but" "Write Changes" {parts_callback WRITE}

    set b [$disk_window button "qbutton" "&Close" {parts_callback QUIT} "default"]
    $b resize "3 -1"

    # fill all this up
    read_disks
    refresh
    
    $disk_window exec

}

#
# callbacks
#

# stub to call update_all with the global args
proc refresh {} {
    global parts
    update_all $parts
}

# update main window display
proc update_all { parts} {
    
    global disk_list
    global size_units
    global sector_size

    set p_sel [$parts select]
    set s_sel [$parts select]
    
    # empty lists and listboxes
    $parts removeAll
    
    update_parts_list
    foreach disk $disk_list {
        
        set sector_size [$disk sector_size]
        recurse_chunk [$disk chunks] "display_chunk" 0
        
    }

    $parts select [min [expr [$parts numberOfItems] - 1] $p_sel]

}

proc cycle_units_callback {hui foo bar} {

    global size_units
    
    set size_units [cycle_size_units $size_units]

    set button [$hui byName "units_but"]
    $button text "Cycle Si&ze Units ($size_units)"
    
    refresh
    
}

proc console_callback {hui foo bar} {
    
    source scripts/diskedit/console.tcl
    open_console $hui

}

#
# we could also implement a "select" callback that would grey out some 
# impossible functions
#
proc parts_callback {hui button userdata} {

    global disk_window
    global parts_list
    global part_to_disk_list
    global disk_list
    global parts
    
    source scripts/diskedit/labeledit.tcl

    set chunk [lindex $parts_list [$parts select]]
    set disk [lindex $part_to_disk_list [$parts select]]
    set slice $chunk
    switch $userdata {
        "ADD" {
            # popup a full "partition settings" window, reusable
            if {[$slice type_text] == "unused" ||\
                    [$slice type_text] == "whole"} {
                global MIN_PART_SIZE
                set free [expr [free_space $slice] * [$disk sector_size]];
                if {$free > $MIN_PART_SIZE} {
                    # stolen from fdiskops.tcl
                    source scripts/diskedit/sliceops.tcl
                    # this has to take care of 'dedicated' too
                    set type [slice_create_dialog $hui $disk_window $slice]
                    if [string compare $type "CANCEL"] {
                        if {[lsearch $type "WHOLE_DISK"] >= 0} {
                            set result [yesno_dialog $hui $disk_window \
                                            "Use Entire Disk" {
Do you want to allocate all the disk to use FreeBSD
with a true partition entry so as to remain cooperative
with any future possible operating systems on the drive?

This would be instead of the ``dangerously dedicated'' 
mode (whole disk) you chose. See that FAQ section for more
information.
}  "40 10"]

                            # compatible is *not* whole disk as old fdisk thought
                            # (see sysinstall/disks.c and msg.c)
                            if {[string compare $result "YES"]} {
                                set force_all 1
                            } else {
                                set force_all 0
                            }
                            
                            $disk All_FreeBSD $force_all
                        } else {
                            $disk Create_Chunk [$slice offset] [lindex $type 0] \
                                [lindex $type 1] \
                                [concat [$slice flags] \
                                     "CHUNK_ALIGN" [lindex $type 2]]
                        }
                        refresh
                    }
                } else {
                    ok_dialog $hui $disk_window "Can't create" \
                        [format {
Not enough space to create a partition.
Partitions must be at least %s in size
%s are available
} \
                             [convert_size_str $MIN_PART_SIZE] \
                             [convert_size_str $free] \
                            ] "40 3"
                }
            } elseif {[$slice type_text] == "freebsd"} {
                    
                # this should return a list containing:
                # type: fs or swap
                # size: sectors int
                global MIN_PART_SIZE
                set free [expr [free_space $slice] * [$disk sector_size]];
                if {$free > $MIN_PART_SIZE} {
                    set ret [part_create_dialog $hui $disk_window $disk $slice]
                    if [string compare $ret "CANCEL"] {
                        set flags {}
                        set type [lindex $ret 0]
                        set size [lindex $ret 1]
                        
                        set new_partition [$disk Create_Chunk_DWIM $slice \
                                               $size $type $flags]
                        if [partition_create_check_created $hui $disk_window \
                                $new_partition $disk $root] {
                        }
                        refresh
                    }
                } else {
                    ok_dialog $hui $disk_window "Can't create" \
                        [format "Not enough space to create a partition.\nPartitions must be at least %s in size\n%s are available"\
                             [convert_size_str $MIN_PART_SIZE] \
                             [convert_size_str $free] \
                            ] "40 3"
                }
            } elseif {[$slice type_text] == "fat"} {
                ok_dialog $hui $disk_window "Can't add"  \
                    "Can't add a partition to a FAT slice" "25 2"
            } elseif {[$slice type_text] == "part"} {
                ok_dialog $hui $disk_window "Can't add"  \
                    "Can't add a partition to a FreeBSD partition!" "25 2"
            } else {
                ok_dialog $hui $disk_window "Can't add"  \
                    [format "unknown part type: %s" [$slice type_text]] "25 2"
            }
        }
        "DELETE" {
            # simple check + delete partition w/ confirm
            if {[$disk Delete_Chunk $chunk]} { # 0 is success here
                switch [$chunk type_text] {
                    "whole" {
                        ok_dialog $hui $disk_window "Delete failed"\
                            "Cannot delete a whole disk!" "35 2"
                    }
                    "unused" {
                        ok_dialog $hui $disk_window "Delete failed"\
                            "Cannot delete unused space!" "35 2"
                    }
                    default {
                        ok_dialog $hui $disk_window "Delete failed"\
                            "Cannot delete chunks" "35 2"
                    }
                }
            }
            refresh
        }
        "RESIZE" {
            # let's not get into this yet
            puts "resize not implemented [you wish!]"
        }
        "BOOT" {
            set flags [$slice flags]
            set bootable_flag_index [lsearch -exact $flags "CHUNK_ACTIVE"]
            if {$bootable_flag_index >= 0} {
                set flags [lreplace $flags $bootable_flag_index $bootable_flag_index]
            } else {
                lappend flags "CHUNK_ACTIVE"
            }
            $slice flags $flags
            refresh
        }
        "EDIT" {
            if {[$slice type_text] == "part"} {
                set ret [part_create_dialog $hui $disk_window $disk $chunk $chunk]

                set type [lindex $ret 0]
                if {$type != "CANCEL"} {
                    set flags {}
                    set size [lindex $ret 1]
                    
                    $chunk flags $flags
                    refresh
                }
            } else {
                ok_dialog $hui $disk_window "Edit failed" \
                    "Can only edit FreeBSD partitions" "30 2"
            }

        }
        "UNDO" {
            # simply re-read the disks and update the display
            set edited 0
            foreach disk $disk_list {
                if {[$disk modified]} {
                    if {[yesno_dialog $hui $disk_window "Really undo?"\
                             "Really undo changes?" "25 2"] == "YES"} {
                        read_disks
                        refresh
                    }
                    set edited 1
                    break
                }
            }
            if {!$edited} {
                ok_dialog $hui $disk_window "Nothing to undo"\
                    "Cannot undo changes because nothing was modified" "35 2"
            }
        }
        "WRITE" {
            # write data back to the disks (note the plural)
            # we inspect the list of disks and check which are modified
            # popup a window with a checkbox/disk, for confirm
            # see fdisk's add part impl
            set edited 0
            foreach disk $disk_list {
                if {[$disk modified]} {
                    # write_disks_confirm should return 1 if disks written
                    if {[write_disks_confirm $hui $disk_window $disk_list]} {
                        # we refresh if disks were modified
                        read_disks
                        refresh
                    }
                    set edited 1
                    break
                }
            }
            if {!$edited} {
                ok_dialog $hui $disk_window "Nothing to write"\
                    "Cannot write changes because nothing was modified" "45 2"
            }
        }
        "TEST" {
            puts [format "list: %s parts selected: %s corresponding list item: %s"\
                      $parts_list [$parts select] [lindex $parts_list [$parts select]]]
        }
        "QUIT" {
            set modified 0
            foreach disk $disk_list {
                if {[$disk modified]} {
                    set modified 1
                }
            }
            if {$modified} {
                if {[yesno_dialog $hui $disk_window "Unsaved changes" \
                         "Are you sure you want to exit the editor?\nUnsaved changes will be lost." "25 3"]
                    == "YES"} {
                    write_disks_confirm $hui $disk_window $disk_list
                    return QUIT;
                }
            } else {
                return QUIT;
            }
        }            
        default {
            puts "unkown callback: $userdata"
        }
    }
    return ""
}
