# global sizes presets
#
# these should be in bytes, *not in sectors*
global MIN_PART_SIZE
# 1 megabyte
set ONE_MEG [expr 1024*1024]
set ONE_GIG [expr 1024 * $ONE_MEG]

# the minimum filesystem size, also minimum partition size
set MIN_PART_SIZE $ONE_MEG
set FS_MIN_SIZE $ONE_MEG

# various defaults
set ROOT_MIN_SIZE [expr 20 * $ONE_MEG]
set ROOT_PARTITION_SIZE [expr 32 * $ONE_MEG]
set PHYSICAL_MEMORY 32
set SWAP_PARTITION_SIZE [expr 16 * $ONE_MEG + $PHYSICAL_MEMORY * $ONE_MEG * 2]
set VAR_PARTITION_MIN_SIZE [expr 30 * $ONE_MEG]
set USR_PARTITION_MIN_SIZE [expr 80 * $ONE_MEG]


proc cycle_size_units {size_units} {

    set size_units_list [list "bytes" "Kb" "Mb" "Gb" "Tb"]

    # current index
    set i [lsearch $size_units_list $size_units]

    # next index (take list as circular)
    set e [expr ($i+1) % ([llength $size_units_list])]
    return [lindex $size_units_list $e]

}

proc convert_size {size} {
    global size_units
    return [r_convert_size $size_units $size]
}

proc convert_size_str {size} {
    global size_units
    return [r_convert_size_str $size_units $size]
}

proc r_convert_size_str {size_units bytes} {

    switch $size_units {
        "" -
        "auto" {
            if [expr $bytes > 1024] {
                return [r_convert_size_str "Kb" $bytes]
            } elseif [expr $bytes > pow (1024, 2)] {
                return [r_convert_size_str "Mb" $bytes]
            } elseif [expr $bytes > pow (1024, 3)] {
                return [r_convert_size_str "Gb" $bytes]
            } elseif [expr $bytes > pow (1024, 4)] {
                return [r_convert_size_str "Tb" $bytes]
            } else {
                return [r_convert_size_str "bytes" $bytes]
            }
        }
    }
    return [format "%.1f %s" [r_convert_size $size_units $bytes] $size_units]
}

proc r_convert_size {size_units size} {

    # make that floating point
    switch $size_units {
        "bytes" {
            # bytes
            return $size
        }
        "Kb" -
        "kb" {
            # kilobytes
            return [expr $size / 1024.0]
        }
        "Mb" -
        "mb" {
            # megabytes
            return [expr $size / pow (1024.0, 2)]
        }
        "Gb" -
        "gb" {
            # gigabytes
            return [expr $size / pow (1024.0, 3)]
        }
        "Tb" -
        "tb" {
            # terabytes
            return [expr $size / pow (1024.0, 4)]
        }
        "" -
        "auto" {
            if [expr $size > 1024] {
                return [r_convert_size "Kb" $size]
            } elseif [expr $size > pow(1024, 2)] {
                return [r_convert_size "Mb" $size]
            } elseif [expr $size > pow(1024, 3)] {
                return [r_convert_size "Gb" $size]
            } elseif [expr $size > pow(1024, 4)] {
                return [r_convert_size "Tb" $size]
            } else {
                return [r_convert_size "bytes" $size]
            }
        }
    }
}

# return a formatted string representing disk geometry
proc disk_geometry_text {disk} {
    # XXX add size of disk in MB
    format "%d cyls/%d heads/%d sectors = %s sectors" \
        [$disk bios_cyl] [$disk bios_hd] [$disk bios_sect] \
        [expr [$disk bios_cyl] * [$disk bios_hd] * [$disk bios_sect]]
        
}

# translated from sysinstall
# return free space
# free = total - sum "not unused" partitions
proc free_space {chunk} {
    
    set size [$chunk size]
    for {set c1 [$chunk part]}\
	{![H::is_null $c1]}\
	{set c1 [$c1 next]} {
            # XXX: it seems we can't use "1" here since unused
            # freebsd space is not seen as "1"
            if {[$c1 type_text] != "unused"} {
                incr size -[$c1 size]
            }
        }
    if {$size < 0} {
        puts stderr "Partitions are larger than actual chunk??"
    }
    return $size
}

# return a convenient string representing the space state
proc free_space_str {free full} {
    global size_units
    if {$free == $full} {
        set free [format "all free: %.1f $size_units" [convert_size $free]]
    } elseif {$free == 0} {
        set free [format "all full: %.1f $size_units" [convert_size $full]]
    } else {
        set free [format "%.1f/%.1f $size_units free" [convert_size $free] [convert_size $full]]
    }
    return $free
}