#!/bin/sh

# $FreeBSD: libh/release/scripts/pkgtools/makepackage.sh,v 1.14 2002/06/03 03:01:31 antoine Exp $
#
# This script is a quick hack to create libh
# package out of existing FreeBSD ports.

usage ( ) {
    echo "Simple package creating tool, contains too many hacks
and doesn't respect @exec and stuff
Usage: $0 /path/to/port /path/to/pkg-output-dir/"
}

cleanup ( ) {
    rm -rf $PKGTMP $SCRIPT
}

clean_exit ( ) {
    cleanup
    exit $1
}

if [ -z $2 ]; then 
    usage
    exit 1
fi

PORTDIR=$1
PKGDIR=$2
if [ ! -d $PORTDIR ] || [ ! -d $PKGDIR ]; then
    usage
    exit 1
fi

PKGTMP=`mktemp -d /var/tmp/pkgXXXXXXXXX`
SCRIPT=`mktemp /tmp/scriptXXXXXXXX`

trap clean_exit 1 2

echo "This is the libh package creator"
echo ""

# Setup tree
echo "Installing port"
(cd $PORTDIR && make clean install NO_PKG_REGISTER=yes PREFIX=$PKGTMP) || \
    (echo "make install failed"; clean_exit)

echo "Generating package list"
PLIST=`cd $PORTDIR && make -V TMPPLIST`
FILELIST=`cat $PLIST | sed -e 's|^@dirrm ||g' -e 's|^@.*||' | sort | uniq -u`
# depends: make package-depends package-depends | ${GREP} -v -E ${PKG_IGNORE_DEPENDS} | sort -u`"

# Some VERSIONING
echo "Collection package names and versions"
PKGNAME=`cd $PORTDIR && make -V PKGNAME`
PORTNAME=`cd $PORTDIR && make -V PORTNAME`
PORTVERSION=`cd $PORTDIR && make -V PORTVERSION`
PORTREVISION=`cd $PORTDIR && make -V PORTREVISION`

echo "Collecting description"
SUMMARY=`cat $PORTDIR/pkg-comment`
DESCRIPTION=`cat $PORTDIR/pkg-descr`
AUTHOR=`cd $PORTDIR && make -V MAINTAINER`

echo "Generating dependencies"
PKGDEP=`cd $PORTDIR && make package-depends`

if [ ! -e $PKGDIR ]; then mkdir $PKGDIR; fi

mkdir $PKGTMP/SYSINSTALL

cat > $PKGTMP/SYSINSTALL/description.tcl <<EOF
name {$PORTNAME}
summary {$SUMMARY}
description {$DESCRIPTION}
author {$AUTHOR}
EOF

# write requires and provides
cat > $PKGTMP/SYSINSTALL/features.tcl <<EOF
proc provides {} {
    return [list [Feature "$PORTNAME" "$PORTVERSION"]]
}

# This is the system feature followed by the dependencies
proc requires {} {
    return [list [Feature "i386" "3.0"] \\
EOF

# this scheme assumes there is alyways only a single - in a pkg name
for pkg in $PKGDEP
do
    version=`echo $pkg | sed 's/^[^-]*-//'`
    name=`echo $pkg | sed 's/-[^-]*$//'`
    echo "[Feature \"$name\" \"$version\"] \\" >> $PKGTMP/SYSINSTALL/features.tcl
done

cat >> $PKGTMP/SYSINSTALL/features.tcl <<EOF 
    	]
}
EOF

cat > $SCRIPT << EOF
set creator [PackageCreator::PackageCreator "$PKGNAME" "$PKGDIR" "/opt" "$PKGTMP"]
\$creator generate
\$creator write_package_file
EOF

../../../compile/text+graphics/tclh.static $SCRIPT

clean_exit