#! /usr/bin/tclh

# $FreeBSD: libh/release/scripts/pkgtools/pkg_install.tcl,v 1.4 2002/06/03 03:04:42 antoine Exp $
#
# This file installs a given package

set filename [lindex $argv 0]

proc usage {} {
    puts stderr "Usage: pkg_install.tcl filename"
}

if {$filename == ""} {
    usage
    exit 1
}

# XXX: this should be set via the command line
set verbose 0

set db [H::null]
if [catch {set db [Database]} err] {
	puts stderr "Can't open pkg db: $err"
	exit 1
    }

if [catch {set stream [Archive::open $filename]} err ] {
    puts stderr "Can't open $filename: $err"
    exit 1
}

set package_stream [PackageStream::open $stream slave]
puts "Package: [$package_stream name]"
puts "Summary: [[$package_stream description] summary]"
puts "Author: [[$package_stream description] author]"
if {$verbose == 1} { 
    puts "Description:"
    puts [[$package_stream description] description]
}

proc view_log {} {
    puts [[Log] text]
}

proc package_action {package_stream action} {
    # No set objects_references [H::objects_references]

    global db
    set old_log_type [Log::gettype]
    Log::type "string"
    if [catch {$package_stream $action $db} err] {
	puts stderr "Can't $action pkg: $err"
	exit 1
    } else {
	puts "Package [$package_stream name] ${action}ed, log:"
	view_log
    }
    $db flush
    [Log] clear
    Log::type $old_log_type
}

# XXX Add upgrade vs. install hooks

package_action $package_stream install

exit 0
