#!/bin/sh

#
# Set default values for display type and binary type.
#
DIS_TYPE="Qt"
BIN_TYPE="dynamic"

#
# Parse command line args.
#
args=`getopt dqst $*`
if [ $? != 0 ]; then
	echo 'Usage: ...'
	echo '	-d	dynamically loaded extension
	echo '	-s	staticly linked extension
	echo '	-q	use Qt display
	echo '	-s	use TVision display
	exit 2
fi
set -- $args
for i; do
	case "$i" in
	-q)
		DIS_TYPE="Qt"
		shift;;
	-t)
		DIS_TYPE="Tv"
		shift;;
	-d)
		BIN_TYPE="dynamic"
		shift;;
	-s)
		BIN_TYPE="static"
		shift;;
	--)
		shift; break;;
	esac
done

#
# Build command path from args.
#
case $DIS_TYPE-$BIN_TYPE in
Qt-static)
	INTERP=HuiQt_static/HuiQt_static;;
Qt-dynamic)
	INTERP=HuiQt_shared/HuiQt.so;;
Tv-static)
	INTERP=HuiTv_static/HuiTv_static;;
Tv-dynamic)
	INTERP=HuiTv_shared/HuiTv.so;;
esac

#
# Locate interpreter
#
for d in \
	../../obj/bin \
	obj/bin \
	; do
	if [ -f $d/$INTERP ]; then
		if [ $BIN_TYPE = static ]; then
			$d/$INTERP setupNew.tcl static $DIS_TYPE
		else
			tclsh8.3 setupNew.tcl $d/$INTERP $DIS_TYPE
		fi
		exit 0
	fi
done

echo "Error: Did not find interpreter"
exit 1
