/*
 * $Id: BuildMake.cc,v 1.4 2002/07/13 10:13:23 okumoto Exp $
 */
#include <iostream>
#include <string>

#include "BuildMake.hh"

/**
 * Construct a Makefile file generator.
 *
 * @param src		Directory containing source files.
 * @param obj		Destination of generated output files.
 * @param n		Noisy Makefile flag
 * @param count		Size if directory array.
 * @param dirs		Array of directories.
 */
BuildMake::BuildMake(
	const string &src, const string &obj,
	bool n,
	int count, const char * const dirs[])
    : srcdir(src), objdir(obj), noisy(n)
{
	if (count == 0) {
		/*
		 * Add current directory to input directories.
		 */
		dirlist.push_back("");
	} else {
		/*
		 * Add list to input directories.
		 */
		for (int i = 0; i < count; ++i) {
			dirlist.push_back(dirs[i]);
		}
        }
}

/**
 * Process all directories for "module.m4" files.
 */
void
BuildMake::process()
{
	/*
	 * Process each "module.m4" file and add any referenced
	 * subdirectories into dirlist.
	 *
	 * XXX hmmm... bad input can cause infinite loop
	 */
	while (dirlist.empty() == false) {
		string dir = dirlist.back();
		dirlist.pop_back();

		mlist[dir] = Module(srcdir, objdir, dir);

		mlist[dir].subdirs(&dirlist);	/* add subdirs to dirlist */
		mlist[dir].process();
	}
}

/**
 * Generate the fragment of the Makefile described by the modules.
 */
void
BuildMake::output(ostream &out) const
{
    out <<
	"# DO NOT EDIT!\n"
	"# This file was automatically generated by BuildMake\n"
	"# DO NOT EDIT!\n"
	"#\n";

    /*
     * Output sections that are libraries.
     */
    for (MList::const_iterator i = mlist.begin(); i != mlist.end(); ++i) {
	if (i->second.uselibs() == false) {
	    i->second.output(out);
	}
    }

    /*
     * Output sections that are programs.
     */
    for (MList::const_iterator i = mlist.begin(); i != mlist.end(); ++i) {
	if (i->second.uselibs() == true) {
	    i->second.output(out);
	}
    }
}
