/*
 * $Id: Library.cc,v 1.4 2002/07/14 06:52:47 okumoto Exp $
 */
#include <string>

#include "Library.hh"

/**
 * Constructor.
 *
 * @param dir	Directory containing module.
 * @param name	name of module.
 */
Library::Library(const string &src, const string &obj,
		 const string &dir, const string &name)
    : ObjectCollection(src, obj, dir, name, "LIB")
{
    target.push_back(objdir + dirname + "/lib" + name + ".a");

    {
	MakeDependency	tmp(target.value());
	tmp.push_back(dfiles.value());
	tmp.push_back(ofiles.value());
	deps.insert(tmp);
    }
    {
	MakeDependency	tmp("dep");
	tmp.push_back(dfiles.value());
	deps.insert(tmp);
    }
    {
	MakeDependency	tmp("clean");
	tmp.push_back("PROG_" + name + "_CLEAN");
	tmp.push_back("PROG_" + name + "_CLEAN_DEP");
	deps.insert(tmp);
    }
    {
	MakeDependency	tmp("clean_dep");
	tmp.push_back("PROG_" + name + "_CLEAN_DEP");
	deps.insert(tmp);
    }
    {
	MakeDependency	tmp("PROG_" + name + "_CLEAN");
	tmp.set_action(
	    "\trm -f " + target.value() + "\n"
	    "\trm -f " + ofiles.value() + "\n"
	    "\trm -f " + cleanfiles.value() + "\n");
	deps.insert(tmp);
    }
    {
	MakeDependency	tmp("PROG_" + name + "_CLEAN_DEP");
	tmp.set_action(
	    "\trm -f " + dfiles.value() + "\n");
	deps.insert(tmp);
    }
}

void
Library::out_assignments(ostream &out) const
{
    out
	<< target.assignment() << "\n"
	<< ofiles.assignment() << "\n"
	<< dfiles.assignment() << "\n"
	<< cflags.assignment() << "\n"
	<< ldflags.assignment() << "\n"
	<< cleanfiles.assignment() << "\n"
	<< "\n";
}

void
Library::out_target(ostream &out) const
{
    for (Dependencies::const_iterator i = deps.begin(); i != deps.end(); ++i) {
	out << i->output();
	out << "\n";
    }
}


/**
 * Perform additional proceessing.
 */
void
Library::process()
{
    MakeDependency	tmp(target.value());

    tmp.set_action(
	"\t@echo archive $@\n"
	"\t@rm -f $@\n"
	"\t@${AR} cq $@ \\\n"
	"\t\t`lorder " + ofiles.value() + " | tsort -q`\n"
	"\t@${RANLIB} $@\n");
    deps.insert(tmp);
}


/**
 * Generate a section of the Makefile fragment.
 */
void
Library::output(ostream &out) const
{
    out_header(out);
    out_assignments(out);
    out_target(out);

    out <<
	".for DFILE in " << dfiles.value() << "\n"
	".if exists(${DFILE})\n"
	".include \"${DFILE}\"\n"
	".endif\n"
	".endfor\n"
	"\n";
}

