#ifndef Module_hh_196c1744
#define Module_hh_196c1744
/*
 * $Id: Module.hh,v 1.5 2002/07/13 10:24:42 okumoto Exp $
 */
#include "ObjectCollection.hh"

#include <stdexcept>
#include <string>
#include <vector>

typedef vector<string>	DirList;	/** directory queue type */

/**
 * Represent a library, program, or subdirectories.  The module is
 * described by the "module.m4" file contained in the directory.
 */
class Module {
    enum ModuleToken {
	tCFILE,
	tCFLAGS,
	tLDFLAGS,
	tLIBRARY,
	tPROGRAM,
	tSUBDIR,
	tSWIGFILE,
	tUSELIB,
	tNULL,
    };

    string	srcdir;		/**< Directory containing source files. */
    string	objdir;		/**< Destination of generated output files. */
    string	dirname;	/**< Directory being processed. */
    string	filename;	/**< File containing module desc */
    DirList	sd;		/**< List of subdirectories */
    ObjectCollection	*oc;

    void parse_line(const string &line, int linenum);

    void decode_action(ModuleToken token, const string &arg, int linenum);

public:
    /**
     * Constructor.
     */
    Module()
	: oc(NULL)
    {
    }

    Module(const string &src, const string &obj, const string &dir);

    /**
     * Destructor.
     */
    ~Module()
    {
    }

    /**
     * Copy list of subdirectories found into 'dq'.
     *
     * @param dq	Queue of directories to be processed.
     */
    inline void
    subdirs(DirList *dq)
    {
	for (DirList::const_iterator i = sd.begin(); i != sd.end(); ++i) {
	    dq->push_back(*i);
	}
    }

    /**
     * Perform additional proceessing.
     */
    inline void
    process()
    {
	if (oc != NULL) {
	    oc->process();
	}
    }

    /**
     * Returns true if the module target depends on other libraries.
     */
    inline bool
    uselibs() const
    {
	return (oc == NULL) ? false : oc->uselibs();
    }

    /**
     * Generate a section of the Makefile fragment.
     */
    inline void
    output(ostream &out) const
    {
	if (oc != NULL) {
	    oc->output(out);
	}
    }
};

#endif /* Module_hh_196c1744 */
