/*
 * $Id: ObjectCollection.cc,v 1.10 2002/08/14 12:43:05 okumoto Exp $
 */
#include "MakeDependency.hh"
#include "ObjectCollection.hh"

#include <string>

void
ObjectCollection::out_header(ostream &out) const
{
	out <<
		"#----------------------------------------\n"
		"# build rules for " << dirname << "\n"
		"#----------------------------------------\n";
}

void
ObjectCollection::add_cfile(const string &base)
{
	const string cfile(srcdir + base + ".c");
	const string ofile(objdir + base + ".o");
	const string dfile(objdir + base + ".d");

	cfiles.push_back(cfile);
	ofiles.push_back(ofile);
	dfiles.push_back(dfile);

	{
		MakeDependency	tmp(ofile);
		tmp.push_back(cfile);
		tmp.set_action(
		    "\t@echo compile " + base + ".c\n"
		    "\t@${CC} ${CFLAGS} " + cflags.value() +
		    " -o " + ofile + " -c " + cfile + "\n");
		deps.insert(tmp);
	}
	{
		string targets("'" + dfile + " " + ofile + "'");

		MakeDependency	tmp(dfile);
		tmp.push_back(cfile);

		tmp.set_action(
			"\t@echo build dependencies for " + base + ".c\n"
			"\t@${BUILD_DEP} " +
				srcdir + " " +
				objdir + " " +
				targets + " " +
				cfile + " " +
				dfile +
				" ${CC} ${CFLAGS} " + cflags.value() + "\n");
		deps.insert(tmp);
	}
}

void
ObjectCollection::add_ccfile(const string &base)
{
	const string ccfile(srcdir + base + ".cc");
	const string ofile(objdir + base + ".o");
	const string dfile(objdir + base + ".d");

	cfiles.push_back(ccfile);
	ofiles.push_back(ofile);
	dfiles.push_back(dfile);

	{
		MakeDependency	tmp(ofile);
		tmp.push_back(ccfile);
		tmp.set_action(
			"\t@echo compile " + base + ".cc\n"
			"\t@${CXX} ${CFLAGS} " + cflags.value() +
			" -o " + ofile + " -c " + ccfile + "\n");

		deps.insert(tmp);
	}
	{
		string targets("'" + dfile + " " + ofile + "'");

		MakeDependency	tmp(dfile);
		tmp.push_back(ccfile);
		tmp.set_action(
			"\t@echo build dependencies for " + base + ".cc\n"
			"\t@${BUILD_DEP} " +
				srcdir + " " +
				objdir + " " +
				targets + " " +
				ccfile + " " +
				dfile +
				" ${CXX} ${CFLAGS} " + cflags.value() + "\n");
		deps.insert(tmp);
	}
}

void
ObjectCollection::add_swigfile(const string &base)
{
	const string swigfile(srcdir + base + ".ii");
	const string ccfile(objdir + base + "_wrap.cc");
	const string ofile(objdir + base + "_wrap.o");
	const string dfile(objdir + base + "_wrap.d");

	cfiles.push_back(swigfile);
	ofiles.push_back(ofile);
	dfiles.push_back(dfile);
	cleanfiles.push_back(ccfile);

	{
		MakeDependency	tmp(ccfile);
		tmp.push_back(swigfile);
		tmp.set_action(
			"\t@echo swig " + base + ".ii\n"
			"\t@${SWIG} ${SWIG_FLAGS} -tcl8 -c++ " +
			" -o " + ccfile + " " + swigfile + "\n");

		deps.insert(tmp);
	}
	{
		MakeDependency	tmp(ofile);
		tmp.push_back(ccfile);
		tmp.set_action(
			"\t@echo compile " + base + "_wrap.cc\n"
			"\t@${CXX} ${CFLAGS} " + cflags.value() +
			" -o " + ofile + " -c " + ccfile + "\n");

		deps.insert(tmp);
	}
	{
		string targets("'" + dfile + " " + ofile + "'");

		MakeDependency	tmp(dfile);
		tmp.push_back(ccfile);
		tmp.set_action(
			"\t@echo build dependencies for " + base + "_wrap.cc\n"
			"\t@${BUILD_DEP} " +
				srcdir + " " +
				objdir + " " +
				targets + " " +
				ccfile + " " +
				dfile +
				" ${CXX} ${CFLAGS} " + cflags.value() + "\n");
		deps.insert(tmp);
	}
}

void
ObjectCollection::uselib(const string &name)
{
	libs.push_back("${LIB_" + name + "_T}");
	ldflags.push_back("${LIB_" + name + "_LDFLAGS}");
}
