#ifndef ObjectCollection_hh_e122b224
#define ObjectCollection_hh_e122b224
/*
 * $Id: ObjectCollection.hh,v 1.5 2002/07/14 08:51:16 okumoto Exp $
 */
#include "MakeDependency.hh"
#include "MakeVariable.hh"

#include <set>
#include <string>
#include <vector>

/**
 * Represents the object files contained within a module.
 */
class ObjectCollection {
    virtual void out_assignments(ostream &out) const = 0;

    virtual void out_target(ostream &out) const = 0;

protected:
    const string	srcdir;	/**< Directory containing source files. */
    const string	objdir;	/**< Destination of generated output files. */
    const string	dirname;/**< Directory containing src. */
    const string	name;	/**< Name of object collection. */

    MakeVariable	target;

    MakeVariable	cfiles;		/**< C or C++ input files */
    MakeVariable	swigfiles;	/**< Swig input files */
    MakeVariable	ofiles;
    MakeVariable	dfiles;
    MakeVariable	cleanfiles;

    MakeVariable	libs;
    MakeVariable	cflags;
    MakeVariable	ldflags;

    typedef multiset<MakeDependency>	Dependencies;
    Dependencies	deps;

    /**
     * Outputs a module section header.
     */
    void
    out_header(ostream &out) const;

public:
    /**
     * Constructor.
     *
     * @param src	Directory containing source files.
     * @param obj	Destination of generated output files.
     * @param dir	Directory containing src.
     * @param n		Name of object collection.
     * @param tag	Type of object collection.
     */
    ObjectCollection(const string &src, const string &obj, const string &dir,
		     const string &n,
		     const string &tag)
	: srcdir(src.empty() ? "" : src + "/"),
	  objdir(obj.empty() ? "" : obj + "/"),
	  dirname(dir), name(n),
	  target(tag, name, "T"),
	  cfiles(tag, name, "C"),
	  swigfiles(tag, name, "SWIG"),
	  ofiles(tag, name, "O"),
	  dfiles(tag, name, "D"),
	  cleanfiles(tag, name, "CLEAN"),
	  libs(tag, name, "L"),
	  cflags(tag, name, "CFLAGS"),
	  ldflags(tag, name, "LDFLAGS")
    {
    }

    /**
     * Destructor.
     */
    virtual ~ObjectCollection() {}

    /**
     * Add c file to source list and its dependancy rules.
     */
    void
    add_cfile(const string &base);

    /**
     * Add c++ file to source list and its dependancy rules.
     */
    void
    add_ccfile(const string &base);

    /**
     * Add swig file to source list and its dependancy rules.
     */
    void
    add_swigfile(const string &base);

    /**
     * Add flags to cfilgs variable.
     */
    void
    append_cflags(const string &flags)
    {
	cflags.push_back(flags);
    }

    /**
     * Add flags to ldflags variable.
     */
    void
    append_ldflags(const string &flags)
    {
	ldflags.push_back(flags);
    }

    /**
     * Add dependancy to a library 'libname'
     */
    void
    uselib(const string &libname);

    /**
     * Perform additional proceessing.
     */
    virtual void process() = 0;

    /**
     * Returns true if the module target depends on other libraries.
     */
    virtual bool uselibs() const = 0;

    /**
     * Generate a section of the Makefile fragment.
     */
    virtual void output(ostream &out) const = 0;
};

#endif /* ObjectCollection_hh_e122b224 */
