/*
 * $Id: Program.cc,v 1.4 2002/07/14 08:21:23 okumoto Exp $
 */
#include <string>

#include "Program.hh"

/**
 * Constructor.
 *
 * @param dir	Directory containing module.
 * @param name	Name of module.
 */
Program::Program(const string &src, const string &obj,
		 const string &dir, const string &name)
    : ObjectCollection(src, obj, dir, name, "PROG")
{
	target.push_back(objdir + dirname + "/" + name);

	{
		MakeDependency	tmp("targets");
		tmp.push_back(target.value());
		deps.insert(tmp);
	}
	{
		MakeDependency	tmp(target.value());
		tmp.push_back(dfiles.value());
		tmp.push_back(ofiles.value());
		tmp.push_back(libs.value());
		deps.insert(tmp);
	}
	{
		MakeDependency	tmp("dep");
		tmp.push_back(dfiles.value());
		deps.insert(tmp);
	}
	{
		MakeDependency	tmp("clean");
		tmp.push_back("PROG_" + name + "_CLEAN");
		tmp.push_back("PROG_" + name + "_CLEAN_DEP");
		deps.insert(tmp);
	}
	{
		MakeDependency	tmp("clean_dep");
		tmp.push_back("PROG_" + name + "_CLEAN_DEP");
		deps.insert(tmp);
	}
	{
		MakeDependency	tmp("PROG_" + name + "_CLEAN");
		tmp.set_action(
			"\trm -f " + target.value() + "\n"
			"\trm -f " + ofiles.value() + "\n"
			"\trm -f " + cleanfiles.value() + "\n");
		deps.insert(tmp);
	}
	{
		MakeDependency	tmp("PROG_" + name + "_CLEAN_DEP");
		tmp.set_action(
			"\trm -f " + dfiles.value() + "\n");
		deps.insert(tmp);
	}
}

void
Program::out_assignments(ostream &out) const
{
	out
		<< target.assignment() << "\n"
		<< ofiles.assignment() << "\n"
		<< dfiles.assignment() << "\n"
		<< cflags.assignment() << "\n"
		<< libs.assignment() << "\n"
		<< cleanfiles.assignment() << "\n"
		<< "\n";
}

void
Program::out_target(ostream &out) const
{
	for (Dependencies::const_iterator i = deps.begin(); i != deps.end(); ++i) {
		out << i->output();
		out << "\n";
	}
}

/**
 * Perform additional proceessing.
 */
void
Program::process()
{
	MakeDependency	tmp(target.value());

	if (libs.size() > 0) {
		tmp.set_action(
			"\t@echo link $@\n"
			"\t@rm -f $@\n"
			"\t${CXX} ${CFLAGS} -o $@ \\\n"
			"\t\t" + ofiles.value() + " \\\n"
			"\t\t`lorder " + libs.value() + " | tsort -q` \\\n"
			"\t\t${LDFLAGS} " + ldflags.list() + "\n");
	} else {
		tmp.set_action(
			"\t@echo link $@\n"
			"\t@rm -f $@\n"
			"\t${CXX} ${CFLAGS} -o $@ \\\n"
			"\t\t" + ofiles.value() + " \\\n"
			"\t\t${LDFLAGS} " + ldflags.list() + "\n");
	}
	deps.insert(tmp);
}

/**
 * Generate a section of the Makefile fragment.
 */
void
Program::output(ostream &out) const
{
	out_header(out);
	out_assignments(out);
	out_target(out);

	out <<
		".for DFILE in " << dfiles.value() << "\n"
		".if exists(${DFILE})\n"
		".include \"${DFILE}\"\n"
		".endif\n"
		".endfor\n"
		"\n";
}

