/*
 * $Id: main.cc,v 1.6 2002/08/11 20:32:28 okumoto Exp $
 */
#include "BuildMake.hh"
#include "Exceptions.hh"

#include <fstream>
#include <stdexcept>
#include <string>

extern "C" {
    #include <unistd.h>	/* getopt() */
}

/**
 * Outputs usage information.
 *
 * @param prog	Name of binary
 */
static void
usage(const string &prog)
{
    cout << "usage: " << prog << " path\n";
}

/**
 * Build a makefile for an entire project.  The resulting Makefile
 * will check all dependencies.
 *
 * It parses the command line arguments, and checks for input files.
 */
int
main(int argc, char *argv[])
{
    const string	prog = argv[0];
    string		srcdir;
    string		dstdir;
    string		outfile;
    bool		noisy = false;

    /*************************************************************************
     * Parse command line args
     *************************************************************************/
    int	c;
    while ((c = getopt(argc, argv, "d:no:s:")) != -1) {
	switch (c) {
	case 'd':
	    dstdir = optarg;	/* set destination directory */
	    break;
	case 'n':
	    noisy = true;	/* generate noisy Makefile */
	    break;
	case 'o':
	    outfile = optarg;	/* output file */
	    break;
	case 's':
	    srcdir = optarg;	/* set source directory */
	    break;
	case '?':
	default:
	    usage(prog);
	    return(1);
	}
    }
    argc -= optind;
    argv += optind;

    /*************************************************************************
     * Process directories
     *************************************************************************/
    try {
	BuildMake	bm(srcdir, dstdir, noisy, argc, argv);
	bm.process();

	if (outfile.empty()) {
		bm.output(cout);
	} else {
		ofstream	out(outfile.c_str());
		bm.output(out);
		out.close();
	}

	return 0;

    } catch (Error &e) {
	cerr << e.what() <<
		" at line " << e.line <<
		" of file " << e.file << endl;
	return 1;

    } catch (exception &e) {
	cerr << e.what() << endl;
	return 1;
    }
}

